.module/ram/abs=2      UHF3A;
.pagelength  59;                    { Works for .LST files with 550C }
.include    <system.k>;

{ Conditional assembly for putting code into EPROM, along with monitor.
  If EPROM, no .exe file is used. So, uhf3.exe is the non-EPROM version.
  1=Being put into EPROM, 0=Being loaded from PC.
  These defines are here and in UHF3_UTL.DSP as I could not get the
  preprocessor to work on include files. }
#define  EPROM  0

{ One and only one CODEC should be used: }
#define  CODEC_1847 1
#define  CODEC_AIC23 0

{ One Board should be used (normally would track CODEC) }
#define  EZKIT 1
#define  DSPX  0

{************************************************************************
 *
 *          EZ-KIT Lite DSP Program for DSP-10 2-meter Transciever
 *			UHF3.DSP Version 32
 *			     17 Nov 2005
 *                           Main Module
 *
 * NOTICE: Copyright (c) 1996-2005
 *         by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This license does not purport to restrict in anyway public domain
 * programs, where they are used for as a basis for routines here.
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm
 *
 * The LMS routines are based on the work of Johan Forrer, KC7WW.
 * Many parts of the program have roots in the applications manuals for
 * the ADSP-2100 Family, published by Analog Devices.  The FFT, sin and
 * log routines are in this category.
 *
 *                   UHF3.DSP
 *  IF Processor for UHF & Microwave Transceivers
 *  48 KHz version
 *  Rev  .0    13 May 96  Initial write - Bob Larkin W7PUA = RSL
 *  Rev  .01   21 June 96 Rewrote all serial communication RSL
 *  Rev  .1    11 Sept96  Added audio standalone RSL
 *  Rev  .2    22 Oct 96  Added overlapping windows, audio FIR RSL
 *  Rev  .3    14 Dec 96  Added KC7WW noise reduction - RSL
 *  Rev  .4    11 Jan 97  Added transmit functions - RSL
 *  Rev 1.0     6 Mar 97  Modified FIR gains, 1st prelim issue - RSL
 *  Rev 1.01   19 Apr 97  Add rf transceiver functions - RSL
 *  Rev 1.02    8 Feb 98  (UU) x2 fixed, added audio gen for audio in.  RSL
 *  Rev 1.1    11 Mar 98  Changed second conversion to non-I/Q  RSL
 *  Rev 1.2     7 Feb 99  Minor changes to rf version to QST issue  RSL
 *  Rev 1.3     6 Mar 99  Updated audio version to ver 1.4 C program  RSL
 *  Rev 1.6    12 Sept99  Aligned versions with C prog.  RSL
 *  Rev 1.7    20 Jan 00  Tidyed up control structure. Added ability to
 *                        reset portions of program. Revised LMS. Added
 *                        wideband FFT. Do not use this or later versions
 *                        with PC programs before 1.7.  RSL
 *  Rev 1.9    28 May 00  Redid triggered 2 second FFT sequencing to stop
 *                        over-runs. Only for LHL7, PUA43 and EME2. RSL
 *  Rev 2.0    18 Oct 00  Added Tukey25 windowing, 1200 and 2400 Hz FFT BW,
 *                        changed escape character, triggered modes for
 *			  narrow FFT's, Moved LMS to block 4 to stop overruns,
 *			  Added programmable denoise beta, decay and length,
 *			  Changed AF filters to 8, gains 1 to 128. RSL
 *              20 Mar 01 Cleaned up comments for Ver 2.0  RSL
 *  Rev 2.1      6 Dec 01 Added transmitter noise, UMON.DSP module. RSL
 *  Rev 2.2     30 Spt 02 Reset on FFT, Transmit Clip all redone  RSL
 *  Rev 2.3     10 Feb 04 Fixed freq response for FM RSL
 *  Rev 2.4,2.5 24 Feb 04 Moved serial transmit to Block 2 for T&R,
 *			  Added shift to Mike Gain ('M' Cmd),
 *			  Added FM Deviation setting, byte 3 of 'S'.  RSL
 *  Rev 2.6     11 Oct 04 Added Power-to-Tone capability ('K' cmd).  RSL
 *  Rev 2A      21 Nov 04 Last status to 0A, added signals to v2f. RSL
 *  Rev 2D      24 Sept05 Paralleled PF1 with PF6 for EZ-Kit
 *  Rev 30      12 Nov 05 Reversed L-R for DSPx RSL
 *  Rev 31      18 Nov 05 Fixed PF1 bug, added pwrl,m,h clr, restored FL1 RSL
 ************************************************************************}

{**********************  DEDICATED DAG INDEX REGISTERS  *******************
 * DAG  index register useage is as follows:
 *   i0, m0, l0:     Primary use by fft; interrupt routines with save
 *   i1, m1, l1:     Used by fft and, with save, by hilbert
 *   i2, m2, l2:     Used only by fft
 *   i3, m3, l3:     Used only by fft
 *   i4, m4, l4:     Primary use by fft; interrupt routines with save
 *   i5, m5, l5:     Used by fft and, with save, by LMS routines
 *   i6, m6, l6:     Autobuffering  tx_buf only
 *   i7, m7, l7:     Autobuffering  rx_buf only
 ********************  INTERRUPTS AND SEC_RECS ****************************
 * SPORT0 generates 48 kHz timed interrupts.
 * Background computation uses primary enviornment.  SPORT0 uses the
 * sec_reg enviornment.
 ************************* FLAG PIN USAGE **********************************
 * FL0	Out	Data for all serial devices
 * FL1	Out	Red LED - On when in SPORT0 interrupt.  For 'scope monitor.
 * FL2	Out	Clock for all serial devices
 *
 * PF0  In	Trigger for triggered reception
 * PF1  Out     For EZ-Kit, Used  as parallel to PF6, in case line was blown
 *              by reversing P101 on DSP-10!
 * PF2	In	Hardware Not Ready
 * PF3	In	PLL's Not Locked
 * PF4	In	PTT - 1 when PTT switch pushed
 * PF5	In	CW Key - 1 when Key down
 * PF6	Out	Latch Enable for 19.68 MHz synthesizer
 * PF7	Out	Latch Enable for shift reg and 126 MHz synthesizer      }

{**************************  CONSTANTS  **********************************}
.const      TAPS1=130;                  { FIR1 - 2.8 KHz LPF, 48kHz rate}
.const      TAPS2=130;                  { FIR2 - 2.8 Interpolation LPF }
.const	    TAPS3=199;			{ FIR3 - Audio fir, 9.6 kHz rate }
.const      H1=71;			{ Num taps in Hilbert #1 FIR filt }
.const      H1P1ON2=36;			{ This is  (H1+1)/2  }
.const      H1M1ON2=35;			{ ...and (H1-1)/2    }
.const      H1M3ON2=34;			{ ...and (H1-3)/2    }

.const      H3=247;			{ Num taps in Hilbert #3 FIR filt }
.const      H3P1ON2=124;		{ This is  (H3+1)/2  }
.const      H3M1ON2=123;		{ ...and (H3-1)/2    }
.const      H3M3ON2=122;		{ ...and (H3-3)/2    }

.const      N=1024;                     { Number of samples to be FFT'd }
.const      Nx2=2048;
.const      Nov2=512;
.const	    Nov4=256;
.const      DATA_BLOCKS=3;		{ UART transmit blocks }
.const      ESC_CH=0x24;		{$}

.const	    AGC_SET_POINT=29200;	{ Log2(10304)=abt 0.63 of full scale }
.const      AGC_GROWTH=16384;		{ Fast attack - 0.5 x error }
.const      AGC_GAIN_MAX=60000;		{ x234 voltage (47 dB) }
.const      AGC_GAIN_MAX_D2=30000;
.const      AGC_GAIN_MIN=256;		{ x1 }
.const      AGC_GAIN_MIN_D2=128;

{****************  SUBROUTINES FROM THIS MODULE  **************************}
.entry	start;

{**************  SUBROUTINES FROM EXTERNAL MODULES  ***********************}
.external	ptr_table;
.external	scramble;
.external	fft_strt;
.external	log_power;
.external	log10_dec;
.external	log10;
.external	dump_intrgrt;
.external	init_util;
.external	out_block_byte;
.external	uart_rcvr;
.external	window;
.external	sin;
.external	arctan;
.external	do_hilbert1;
.external	do_hilbert2;
.external	do_hilbert4;
.external	do_fir1i;
.external	do_fir1q;
.external	interp_filter;
.external	audio_fir;
.external	fir10hp;
.external	fir6lp1;
.external	fir6lp2;
.external	fir1200;
.external	fir2400;

#if CODEC_1847
.external	next_cmd;
#endif

.external	LMS_init;
.external	do_LMS;
.external	do_autonotch;
.external	wfir1i;
.external	wfir1q;
.external	wfir2i;
.external	wfir2q;
.external	getrnd;

{********************   DATA MEMORY     *******************************}
.external	tx_buf, rx_buf;
.external	idata, qdata;
.external	audio_data;
.external	audio_fir_data;
.external	h1delay, h1data;
.external	h2delay, h2data;
.external	tx_buf_len, tx_buf_ptr, tx_buf_word_len;
.external	esc_a;
.external	status_buf;
.external	xmit_status_buf;
.external	cmd_buf;
.external	fft_buff_count;
.external	do_window;
.external	iq_lpf;
.external	integrate_count;
.external	mode_select;
.external	ssb_select;
.external	i0_sav, m0_sav, l0_sav;
.external	m1_sav;
.external	i4_sav, m4_sav, l4_sav;
.external	fir1i_i0_sav, fir1q_i0_sav, fir2_i0_sav, fir3_i0_sav;
.external	win_i4_sav, win_m4_sav, win_l4_sav;
.external	if_dphase, if_phase;
.external	isave, qsave, audsav;
.external	audio_fir_on, audio_fir_select, audio_fir_gain;
.external	h1delay_i0_sav, h1data_i0_sav, 	h2delay_i0_sav, h2data_i0_sav;
.external	block_count;
.external	divide_count;
.external	UART_tx_data_block, UART_tx_ptr, UART_tx_ch_count;
.external	UART_tx_cur_bit, UART_tx_cur_data, UART_tx_word_len;
.external	UART_tx_byte_num, UART_tx_esc, UART_tx_resync_count;
.external	UART_tx_data_done, UART_tx_buf_select;
.external	UART_tx_data_buf0, UART_tx_data_buf1;
.external	UART_rx_ctr, UART_rx_cur_bit, UART_rx_cur_byte;
.external	UART_rx_cmd_count, UART_rx_cmd_buf_ptr;
.external	serial_timeout;
.external	serial_error;
.external	stat_flag;
.external	inptr0, inptr1, dataptr;
.external	get_data_block;
.external	beta, denoise_length, denoise_lengthm1, decay;
.external	beta_aut, notch_delay, decay_aut;
.external	squelch_noise;

.var/dm/abs=H#0000	inplacereal [N], inplaceimag [N];
.global		inplacereal, inplaceimag;
.var/dm/circ    inputreal0[N], inputreal1[N], inputreal2[N];
.global		inputreal0, inputreal1, inputreal2;
.var/dm		groups, bflys_per_group, node_space, blk_exponent;
.global         groups, bflys_per_group, node_space, blk_exponent;
.var/dm		out_log[Nov2];		{ 0 to 255 log, or 16 bit }
.global         out_log;		{  depending on dm(data_format }
.var/dm		log_offset;		{ Added to 8 bit log before output }
.global		log_offset;
.var/dm		power_lo[Nov2];	{ Integrated power, lo 16 bits }
.global         power_lo;
.var/dm		power_med[Nov2];	{   middle 16 bits }
.global         power_med;
.var/dm		power_hi[Nov2];	{   and hi 8 bits }
.global         power_hi;
.var/dm		power_exp;		{ Power of 2 power data right shifted }
.global		power_exp;
.var/dm		cur_state;		{ 0 to 11 for 512 byte data gathering }
{ data_collect is used by ISR to control input to FFT. Used in trig'd modes }
.var/dm		data_collect;		{ Flag: 0=Idling, 1=Normal gather }
.var/dm		r_if_gain;
.var/dm		xmit_power;		{ 0 to 32767, multiply for cw, ssb }
.var/dm		mike_gain;		{ 0 to 32767, g/4 before lpf }
.var/dm		mgain_shift;		{ 0, 1, 2, 3 bits of shift, mike gain }
.var/dm		af_gain;		{ Voltage gain, modified by af_shift }
.var/dm		af_shift;		{ Number to put in SE for 6dB steps }
.var/dm		squelch;		{ 0 to 32767 from squelch control }
.var/dm		fm_dev;			{ Transmitter FM dev. 12996->5 kHz }
.var/dm		q_correction;
.var/dm		ad_big;			{ Biggest input, for pc meter display }
.var/dm		dac_big;
.var/dm		del_clip_big;
.var/dm		audio_LMS_on;		{ 0=Off, 1=LMS denoise, -1=autonotch }
.var/dm		data_format;		{ See 9 command }
.global		data_format;
.var/dm		out_steer;		{ 0= standard, 1= Rcvr to Left DAC }
.var/dm		t_r_control;
{ t_r_control sets sequencing and cw keying.
  Bit 0 (value 1)  =1 for dsp xmit.
  Bit 1 (value 2)  =1 for sending of xmit status 
  Bits 2, 3 for unused control.
  Bit 4 (value 16) =1 for cw key down }
.var/dm		sr_data;		{ 16 bits to be clocked out to SR}
.var/dm		sr_count;		{ 16 down to 0. 0 indicates no data }
.var/dm		dclevh, dclevl;		{ Running average of input offset }
.var/dm		last_ph;		{ For fm demod, last phase }
.var/dm		sdtone_ph, sdtone_dph;	{ CW sidetone }
.var/dm		sdtone_ampl;
.var/dm		agc_decay;		{ Proportion of AGC error }
.var/dm		agc_gain;		{ 9.7 format 1 to 255 }
.var/dm		audio_env;
.var/dm		save_out;		{ Temp storage for fm de-emphasis }
.var/dm		save3to4;		{ Block 3 to block 4 }
.var/dm		pwrh, pwrm, pwrl;	{ Accumulated power estimate }
.var/dm		temp1, temp2, temp3;	{ Local use in routines }
.var/dm		need_reset;		{ Communications between interrupt
					  and reset_here routine }
.var/dm		need_gomon;
.var/dm		ar_value;		{ AR when going back to monitor }
.var/dm		delay_right;		{ Amount of delay on right ch, 0,2047 }
.var/dm/circ	delay[2048];		{ The delay line, use all or part }
.var/dm		del_i0_sav;		{ Storage when not in interrupt }

.var/dm		lsb_save, usb_save;
.var/dm		fft_fcn;		{ Commands from bytes 1&2 of 'F' cmd }
.global		fft_fcn;
.var/dm		time_tag;		{ Even seconds into minute x4 }
.var/dm		sav_time_tag;		{ Spare copy to survive next trigger }
.var/dm		random_lev;

{ .var/dm		nb_gain, nb_delay, nb_decay;  <<<<<<<<<<<<}

.var/dm		xlast, ylast;		{ IIR for NBFM pre-emphasis }
.var/dm		fm_ave_l, fm_ave_h;	{ Center freq for PC status+3, +4 }

.var/dm		do_v2f;			{ See set_p2t: for bit definitions }
.global		do_v2f;
.var/dm		t_v2f_h, t_v2f_l;	{ 32-bit Decay multiplier for v2f }
.var/dm		omt_v2f_h, omt_v2f_l;	{ 32-bit 1-t_v2f (gain for new entry) }
.var/dm		offset_v2f;		{ Offset in frequency, 18.75 step }
.var/dm		k_v2f;			{ Gain constant for v2f }
.var/dm		basef_v2f;		{ Base freq such as 3072 for 450 Hz }
.var/dm		phase_v2f;		{ Phase for v2f sine wave }
.const		HP_A1=-18762;
.const		HP_B0=27054;
.var/dm		last_hp_x, last_hp_y;
.var/dm		v_v2f1, v_v2f2, v_v2f3, v_v2f4, v_v2f5; { For v2f }
.var/dm		pwr_v2f_h, pwr_v2f_l;	{ 32-bit power sum over 5 samples }
.var/dm		rng_pave_h, rng_pave_l;	{ 32-bit running power sum }
.var/dm		save_db_v2f;		{ Log of rng_pave_h, _l }
.var/dm		save_if;		{ I-F sample, saved forv2f processing }
.var/dm		v2f_bin_low, v2f_bin_high; { Bin limits for FFT }
.global		v2f_bin_low, v2f_bin_high;
.var/dm		v2f_pwrl, v2f_pwrh;	{ Used in FFT for bins }
.global		v2f_pwrl, v2f_pwrh;
.var/dm		v2f_pkl, v2f_pkh;	{ Used in FFT for bins }
.global		v2f_pkl, v2f_pkh;
.var/dm		v2f_bin_ready;		{ Data collected 18.75/sec }
.global		v2f_bin_ready;
.var/dm		v2f_save_freq;		{ Between updates, for signal source }
.var/dm		v2f_af_level;
		{ For FM transmitting of modulation tone(s) }
		{ ttn_on: Bit0=tone 1, Bit1=tone 2; Bit2=tone 1 and voice }
.var/dm		ttn_on;
.var/dm		ttn_level1, ttn_freq1, ttn_ph1;
.var/dm		ttn_level2, ttn_freq2, ttn_ph2;
.var/dm		getn_data;
.var/dm tmp0, tmp1;

{*****************  PROGRAM MEMORY  *****************}
.var/pm/circ    twid_real [Nov2];
.init           twid_real: <twid_rea.dat>;
.var/pm/circ    twid_imag [Nov2];
.init           twid_imag: <twid_ima.dat>;
.global         twid_real, twid_imag;
.var/pm/circ	cf2[12];
.init		cf2:    1121, 1799, 2525, 3202, 3725, 4011,
			4011, 3725, 3202, 2525, 1799, 1121;
.var/pm/circ	cf12[12];
.init		cf12:    528,  909, -1486, -2455, 4544, 14344,
		       14344, 4544, -2455, -1486,  909,   528;

.external	do_hilbert3;
.external	fir_xmt_audio;
.external	fir_xmt_cw;

.external	fir4_i0_sav, fir5_i0_sav;
.external	xmt_audio_data, xmt_cw_data;
.external	h3delay, h3data;

{****************      MACROS      *************************}
{ do512(%0,%1,%2,%3) - Standard processing done on 1024 point block.  Because
  of overlapping windows this occurs every 512 data points, or 18.75 times
  per second.  Parameters %1 and %2 set the data pointers to fill
  the buffers inputreali, i=0,1,2.  These are sequenced through to overlap
  the windows.  Two are for data gathering and the third is for fft input  }

.macro do512(%0, %1, %2, %3);
	ax1=%0;
	dm(cur_state)=ax1;
	call get_data;			{ Sit here until 512 points collected}
	ax1=%2;				{ For starting data block }
	dm(%1)=ax1;
	ax1=%3;				{ Buffer just filled, ready }
	dm(dataptr)=ax1;
	call window;			{   for windowing }
	call scramble;			{   and fft }
	call fft_strt;
	ar=dm(t_r_control);		{ Bit 0 = 1 is xmt }
	ay0=1;
	ar=ar and ay0;
	if gt call xmt_on;		{ Hold up reception }
.endmacro;

{ DO512R(  ) - Identical with do512( ) except that there is no interruption
  allowed for radio transmission.  }
.macro do512r(%0, %1, %2, %3);
	ax1=%0;
	dm(cur_state)=ax1;
	call get_data;
	ax1=%2;				{ For starting data block }
	dm(%1)=ax1;
	ax1=%3;				{ Buffer just filled, ready }
	dm(dataptr)=ax1;
	call window;			{   for windowing }
	call scramble;			{   and fft }
	call fft_strt;
.endmacro;

{ ADD_POINT   Interpolation requires adding 4 data points (zeros) for
  each actual data point.  This code adds a zero each time.              }
  .macro add_point;
	i0=dm(fir2_i0_sav);  m0=1;  l0=TAPS2;
	ax0=0;
	dm(i0, m0)=ax0;
	dm(fir2_i0_sav)=i0;
.endmacro;

{***************  Interrupt vector table  ******************}

{ This is in the small module UHF3 for the first 3 words:
        jump start;			01: Reset
	.var/pm/ram/abs=1	tbl;
	.init			tbl:^ptr_table;
  This module, UHF3A, starts at word 2.
}

{ Configuration - allows the PC to know what hardware is being used.
  Bits 3 to 0 are not used.
  CODEC Bits 6,5,4   000=1847  001=AIC23
  Board Bits 9,8,7   000=EZKIt 001=DSPx  (presently tracks CODEC)
  EPROM Bit 10        0=No EPROM  1=EPROM load  (always EZKit)
  Bits 15 to 11       Not assigned

  To be sure this is in location 2 of PM, the ax0=<data> instruction
  is used. The LS 4 bits of the op code are always 0 (for ax0) and data
  bits occupy positions 19 to 4 of the op code. Keeping data bit 3 to 0
  as zeros will make the data align with the middle byte of PM.
  The op code does have bit 22 set, which can be ignored.  All 24 bits  of
  the op code are shown in the comment areas.
}
#if (CODEC_1847 & EZKIT & ~EPROM)
					{     15                  0<-Data bits}
ax0 = 0X0000;				{ 0100 00000 0 000 000 00000000 }
#endif
#if (CODEC_1847 & EZKIT & EPROM)
ax0 = 0X0400;				{ 0100 00000 1 000 000 00000000 }
#endif
#if CODEC_AIC23
ax0 = 0X0090;				{ 0100 00000 0 001 001 00000000 }
#endif

	{ ax0=0X2A0; puts the version number, 2A, at a hex byte split; this
	  lets one see what the version is from the .EXE and .HEX files. }
	ax0=0X2A0;			{was rti; PM location 03 <<<<<TEST }

        rti;         rti; rti; rti;     {04: IRQ2 }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }

#if CODEC_1847
	ar = dm(stat_flag);		{10: SPORT0 tx }
        ar = pass ar;			{  see if needs initialization }
        if eq rti;
        jump next_cmd;
#endif
#if CODEC_AIC23
	rti; rti; rti; rti;		{ 10: SPORT0 tx }
#endif

	jump input_samples;             {14: SPORT0 rx }
                     rti; rti; rti;
#if EPROM
        jump doirqe; rti; rti; rti;     {18: IRQE }
#else
        rti;         rti; rti; rti;     {18: IRQE }
#endif
        rti;         rti; rti; rti;     {1c: BDMA }
	rti;         rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
	rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }

{***************************  START  ************************************
 *  Initialize process and drop into main loop
 ************************************************************************}
start:	imask=b#0000010000;              { Leave IRQE available }
	ax0=0; dm(need_reset)=ax0;	{ Don't want reset during local_init }

	call local_init;
	call prog19mhz;
	call init_util;
	call LMS_init;

{****************     Main Program Loop    ******************************
 *  Main processing loop.  Does fft's and other processing.
 *  This loop is interrupted at a 48KHz rate to send and receive Codec data.
 *  There are 12 data gathering states (and fft's) before the log powers are
 *  sent to the host pc.  This takes 640 msec.  In order to have overlapping
 *  windows there are three input buffers, two being filled and one being
 *  processed for fft data.  Two versions: again() repeats endlessly, whereas
 *  trig2sec() waits for a trigger and completes 36 data/fft sets
 *  in 1.92 seconds and waits for a new trigger that would ordinarily occur
 *  at a 2 sec period.  Version set by bit 7 of mode_select.
 *************************************************************************}
again:	ar=dm(need_gomon);		{ See if ISR request monitor }
	none=pass ar;
	if eq jump ag1;			{ No request }
	ar=dm(ar_value);		{ AR is reason for going }
	jump 0x3800;			{ Start of monitor }

ag1:	ax0=dm(fft_fcn);		{ See if measurements are continuous }
	ar=tstbit 9 of ax0;
	if ne jump trig2sec;		{ No, triggered }
	ar=tstbit 0 of ax0;
	if ne jump setof3;
	ar=tstbit 1 of ax0;
	if ne jump setof6;

	ax0=1;
	dm(getn_data)=ax0;		{ A flag }

ag1a:	do512(0,  inptr0, ^inputreal0, ^inputreal1);
	do512(1,  inptr1, ^inputreal1, ^inputreal2);
	do512(2,  inptr0, ^inputreal2, ^inputreal0);
	do512(3,  inptr1, ^inputreal0, ^inputreal1);
	do512(4,  inptr0, ^inputreal1, ^inputreal2);
	do512(5,  inptr1, ^inputreal2, ^inputreal0);
	do512(6,  inptr0, ^inputreal0, ^inputreal1);
	do512(7,  inptr1, ^inputreal1, ^inputreal2);
	do512(8,  inptr0, ^inputreal2, ^inputreal0);
	do512(9,  inptr1, ^inputreal0, ^inputreal1);
	do512(10, inptr0, ^inputreal1, ^inputreal2);
	do512(11, inptr1, ^inputreal2, ^inputreal0);
	ax0=0;
	dm(getn_data)=ax0;		{ A flag }

	ar=dm(need_reset);		{ See if reset requested by PC }
	none=pass ar;
	if ne call reset_here;		{ Yes, go do it }
	ax0=0XFFFF;
	dm(status_buf+14)=ax0;		{ Mark as coming from again() }
	call send12data;
	jump again;

setof6:	ax0=1;
	dm(getn_data)=ax0;		{ A flag }

	do512(0,  inptr0, ^inputreal0, ^inputreal1);
	do512(1,  inptr1, ^inputreal1, ^inputreal2);
	do512(2,  inptr0, ^inputreal2, ^inputreal0);
	do512(3,  inptr1, ^inputreal0, ^inputreal1);
	do512(4,  inptr0, ^inputreal1, ^inputreal2);
	do512(5,  inptr1, ^inputreal2, ^inputreal0);
	ax0=0;
	dm(getn_data)=ax0;		{ A flag }
	ar=dm(need_reset);		{ See if reset requested by PC }
	none=pass ar;
	if ne call reset_here;		{ Yes, go do it }
	ax0=0XFFFF;
	dm(status_buf+14)=ax0;		{ Mark as coming from again() }
	call send12data;
	jump again;

setof3:	ax0=1;
	dm(getn_data)=ax0;		{ A flag }
	do512(0,  inptr0, ^inputreal0, ^inputreal1);
	do512(1,  inptr1, ^inputreal1, ^inputreal2);
	do512(2,  inptr0, ^inputreal2, ^inputreal0);
	ax0=0;
	dm(status_buf+14)=ax0;		{ Mark as coming from again() }
	call send12data;
	do512(3,  inptr1, ^inputreal0, ^inputreal1);
	do512(4,  inptr0, ^inputreal1, ^inputreal2);
	do512(5,  inptr1, ^inputreal2, ^inputreal0);

	ax0=0;
	dm(getn_data)=ax0;		{ A flag }
	ar=dm(need_reset);		{ See if reset requested by PC }
	none=pass ar;
	if ne call reset_here;		{ Yes, go do it }
	ax0=0XFFFF;
	dm(status_buf+14)=ax0;		{ Mark as coming from again() }
	call send12data;
	jump again;

{ Triggered 2 second data collection and processing.  Enabled by bit 9 of
  the mode_select word.  Collects 36 1024 word blocks of data in 2 seconds and
  sends them to the pc as 3 logged data blocks.
  Triggers can come from either (a) 2 second pulses at PF0, or (b) 2 second
  spaced commands from the PC, sent in the 'F' FFT command. (See set_fft_fcn: )
  }
trig2sec:
	ar=dm(need_reset);		{ See if reset requested by PC }
	none=pass ar;
	if ne call reset_here;		{ Yes, go do it }

	{ We accept requests for radio transmit here:   }
	ar=dm(t_r_control);		{ Bit 0 = 1 is radio xmit }
	ay0=1;
	ar=ar and ay0;
	if gt call xmt_on;		{ Hold up reception }

	ax1=dm(fft_fcn);		{ See if Triggered, or Free run }
	ar=tstbit 9 of ax1;
	if eq jump again;		{ FFT is free run }
	ar=tstbit 11 of ax1;		{ See if hardware or software trig }
	if eq jump t2b;			{ Jump if software trigger }

	{ Next watch for trigger in hardware, but also for changes in
	  FFT trigger format that indicate trigger is never coming }
t2a:	ax1=dm(fft_fcn);		{ New copy--it may have changed }
	ar=tstbit 9 of ax1;		{ See if PC cancelled trig measurement}
	if eq jump again;		{ No longer triggered }
	ar=tstbit 11 of ax1;		{ Maybe not hardware any more }
	if eq jump trig2sec;		{ Yup, it is now software }

	ax0=dm(PFDATA);			{ Hardware trigger }
	ar=tstbit 0 of ax0;		{ Look for 2 second trigger }
	if eq jump t2a;			{ Keep waiting }
	jump t2c;			{ Hardware trigger was received }

	{ Next watch for trigger in software, but also for changes in
	  FFT trigger format that indicate trigger is never coming.
	  Software trigger is bit 10 of fft_fcn. }
t2b:	ax1=dm(fft_fcn);		{ New copy--it may have changed }
	ar=tstbit 9 of ax1;		{ See if PC cancelled trig measurement}
	if gt jump t2bb;		{ No, still triggered }
	{ Can't make up their minds }
	jump again;			{ Leave }

t2bb:	ar=tstbit 11 of ax1;		{ Maybe not software any more }
	if ne jump trig2sec;		{ Yup, it changed }
	ar=tstbit 10 of ax1;		{ Looking for software trigger }
	if eq jump t2b;			{ Keep looking }

	{ Fall through here if a trigger has been received }
t2c:	ar=clrbit 10 of ax1;		{ Require a new command each 2 sec }
	dm(fft_fcn)=ar;			{  by resetting it here }

        call dump_intrgrt;		{ Clear power sums to 0 }
	{ For FM trig, clear power sums }
	ax0=0; dm(pwrl)=ax0; dm(pwrm)=ax0; dm(pwrh)=ax0;

	{ Save the time tag so if there is a new trigger before the last
	  of the triggered data is sent, we will get the right tag on the data.
	}
	ax0=dm(time_tag);
	dm(sav_time_tag)=ax0;

	ar=tstbit 14 of ax1;		{ See if No FFT at all }
	if ne jump tnofft;		{ Used to get status only }
	ar=tstbit 1 of ax1;		{ Check if a 4800 Hz sample rate }
	if ne jump t48;			{ Yes it is, do 17 collection periods }
	ar=tstbit 0 of ax1;		{ Check if a 2400 Hz sample rate }
	if ne jump t24;			{ Yes it is, do 8 collection periods }

	ax0=1;
   { SET FL1; }	 			{  For checking trigger timing }
	dm(getn_data)=ax0;		{ A flag }
	{ ax0=1; }				{ A 2 sec trig for 9600 samp/s rcvd }
	dm(get_data_block)=ax0;		{ Ready to collect 512 data points }
	dm(data_collect)=ax0;		{ Start gathering again }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }

	{ The following was a macro expansion, modified here for the first
	  time through to not fft the data left over from the 35th pass
	  during the previous 2 seconds:
 	    do512(0, inptr0, ^inputreal0, ^inputreal1);
	}
	{This pointer is for the next 512 point block }
	ax1=^inputreal0;		{ For starting data block }
	dm(inptr0)=ax1;
	ax1=1;
	dm(cur_state)=ax1;
	call get_data;			{ Sit here until 512 points collected}
	ax1=^inputreal1;		{ For starting data block }
	dm(inptr1)=ax1;
	ax1=^inputreal2;		{ Buffer part filled, NO FFT }
	dm(dataptr)=ax1;
	do512(2,   inptr0, ^inputreal2, ^inputreal0);
	do512(3,   inptr1, ^inputreal0, ^inputreal1);
	do512(4,   inptr0, ^inputreal1, ^inputreal2);
	do512(5,   inptr1, ^inputreal2, ^inputreal0);
	do512(6,   inptr0, ^inputreal0, ^inputreal1);
	do512(7,   inptr1, ^inputreal1, ^inputreal2);
	do512(8,   inptr0, ^inputreal2, ^inputreal0);
	do512(9,   inptr1, ^inputreal0, ^inputreal1);
	do512(10,  inptr0, ^inputreal1, ^inputreal2);
	do512(11,  inptr1, ^inputreal2, ^inputreal0);

	ax0=dm(fft_fcn);		{ See if doing 12 or 36 sets per send }
	ar=tstbit 12 of ax0;
	if ne jump do36a;		{ Jump for 36 per send }

	ay0=b#0000000011111100;		{ Make sure only the right 6 bits }
	{ This is the first of the 3 data sets, and so it has bits 1&0 as 0 }
	ax0=dm(sav_time_tag);
	ar=ax0 and ay0;
	dm(sav_time_tag)=ar;
	dm(status_buf+14)=ar;		{ Mark time tag }
	{ Now, log the sums, clear them and start sending the data. }
	call send12data;
do36a:	{ And back to fft's }
	do512(12,  inptr0, ^inputreal0, ^inputreal1);
	do512(13,  inptr1, ^inputreal1, ^inputreal2);
	do512(14,  inptr0, ^inputreal2, ^inputreal0);
	do512(15,  inptr1, ^inputreal0, ^inputreal1);
	do512(16,  inptr0, ^inputreal1, ^inputreal2);
	do512(17,  inptr1, ^inputreal2, ^inputreal0); 
	do512(18,  inptr0, ^inputreal0, ^inputreal1);
	do512(19,  inptr1, ^inputreal1, ^inputreal2);
	do512(20,  inptr0, ^inputreal2, ^inputreal0);
	do512(21,  inptr1, ^inputreal0, ^inputreal1);
	do512(22,  inptr0, ^inputreal1, ^inputreal2);
	do512(23,  inptr1, ^inputreal2, ^inputreal0);

	ax0=dm(fft_fcn);		{ See if doing 12 or 36 sets per send }
	ar=tstbit 12 of ax0;
	if ne jump do36b;		{ Jump of 36 per send }
	{ Here with bits 0,1 of time_tag = 0 }
	ar=dm(sav_time_tag);
	ay0=1;
	ar=ar or ay0;
	dm(status_buf+14)=ar;		{ Mark time tag }
	call send12data;
do36b:	do512(24,  inptr0, ^inputreal0, ^inputreal1);
	do512(25,  inptr1, ^inputreal1, ^inputreal2);
	do512(26,  inptr0, ^inputreal2, ^inputreal0);
	do512(27,  inptr1, ^inputreal0, ^inputreal1);
	do512(28,  inptr0, ^inputreal1, ^inputreal2);
	do512(29,  inptr1, ^inputreal2, ^inputreal0);
	do512(30,  inptr0, ^inputreal0, ^inputreal1);
	do512(31,  inptr1, ^inputreal1, ^inputreal2);
	do512(32,  inptr0, ^inputreal2, ^inputreal0);
	do512(33,  inptr1, ^inputreal0, ^inputreal1);
  	do512(34,  inptr0, ^inputreal1, ^inputreal2);
{	do512(35,  inptr1, ^inputreal2, ^inputreal0);  Expanded below }
{ Again, expand the last do512() to get data collection stopped }
{ To pick up some time, we will short change the last 12 by one 512 block: }
{ This 512:
	Inputs last  512 to inputreal1 (setup in 33)
	Inputs first 512 to inputreal2 (setup in 34)
	FFT's input real0
}
	ax1=35;				{ %0 }
	dm(cur_state)=ax1;
	call get_data;			{ Finish up last data collect }
	ax0=0;
	dm(data_collect)=ax0;		{ Stop ISR from collecting data }
	ax1=^inputreal0;		{ %3 - Buffer filled duing 34, 35 }
	dm(dataptr)=ax1;
	call window;			{   ready for windowing }
	call scramble;			{   and fft }
	call fft_strt;

	ar=dm(sav_time_tag);
	ay0=2;
	ar=ar or ay0;
	dm(status_buf+14)=ar;		{ Mark time tag }
	call send12data;		{ Logs data & sets up status, xmit }

  {  RESET FL1;  }			{  For checking trigger timing }

	ax0=0;
	dm(getn_data)=ax0;		{ A flag }
	jump trig2sec;

{ Triggered measurement for 4800 Hz sample rate, 1.92 second collection.
  Overlapping data periods:

    Trig |--1--|--3--|--5--|--7--|--9--|-11--|-13--|-15--|-17--| Stop
            |--2--|--4--|--6--|--8--|-10--|-12--|-14--|-16--|

  Unlike the 9600 triggered measurements this does not divide into thirds
  and is sent as a block after 1.92 seconds.
}
t48:	ax0=1;
	dm(getn_data)=ax0;		{ A flag }
	ax0=1;
	dm(get_data_block)=ax0;		{ Ready to collect 512 data points }
	dm(data_collect)=ax0;		{ Start gathering again }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }

	{ The following was a macro expansion, modified here for the first
	  time through to not fft the data left over from the 17th pass
	  during the previous 1.92 seconds:
 	    do512(0, inptr0, ^inputreal0, ^inputreal1);
	}
	{This pointer is for the next 512 point block }
	ax1=^inputreal0;		{ For starting data block }
	dm(inptr0)=ax1;
	ax1=1;
	dm(cur_state)=ax1;
	call get_data;			{ Sit here until 512 points collected}
	ax1=^inputreal1;		{ For starting data block }
	dm(inptr1)=ax1;
	ax1=^inputreal2;		{ Buffer part filled, NO FFT }
	dm(dataptr)=ax1;
	do512(2,   inptr0, ^inputreal2, ^inputreal0);
	do512(3,   inptr1, ^inputreal0, ^inputreal1);
	do512(4,   inptr0, ^inputreal1, ^inputreal2);
	do512(5,   inptr1, ^inputreal2, ^inputreal0);
	do512(6,   inptr0, ^inputreal0, ^inputreal1);
	do512(7,   inptr1, ^inputreal1, ^inputreal2);
	do512(8,   inptr0, ^inputreal2, ^inputreal0);
	do512(9,   inptr1, ^inputreal0, ^inputreal1);
	do512(10,  inptr0, ^inputreal1, ^inputreal2);
	do512(11,  inptr1, ^inputreal2, ^inputreal0);
	do512(12,  inptr0, ^inputreal0, ^inputreal1);
	do512(13,  inptr1, ^inputreal1, ^inputreal2);
	do512(14,  inptr0, ^inputreal2, ^inputreal0);
	do512(15,  inptr1, ^inputreal0, ^inputreal1);
	do512(16,  inptr0, ^inputreal1, ^inputreal2);
	do512(17,  inptr1, ^inputreal2, ^inputreal0);

	ay0=b#0000000011111100;		{ Make sure only the right 6 bits }
	ax0=dm(sav_time_tag);
	ar=ax0 and ay0;
	dm(sav_time_tag)=ar;
	dm(status_buf+14)=ar;		{ Mark time tag }
	{ Now, log the sums, clear them and start sending the data. }
	call send12data;		{ It's really 17! }
	ax0=0;
	dm(getn_data)=ax0;		{ A flag }
	jump trig2sec;

{ Triggered measurement for 2400 Hz sample rate, 1.92 second collection.
  Overlapping data periods:

    Trig |----1----|----3----|----5----|----7----|      Stop
              |----2----|----4----|----6----|----8----|

  Unlike the 9600 triggered measurements this does not divide into thirds
  and is sent as a block after 1.92 seconds.
}
t24:	ax0=1;
	dm(getn_data)=ax0;		{ A flag }
	{ ax0=1; }
	dm(get_data_block)=ax0;		{ Ready to collect 512 data points }
	dm(data_collect)=ax0;		{ Start gathering again }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }

	{ The following was a macro expansion, modified here for the first
	  time through to not fft the data left over from the 8th pass
	  during the previous 1.92 seconds:
 	    do512(0, inptr0, ^inputreal0, ^inputreal1);
	}
	{This pointer is for the next 512 point block }
	ax1=^inputreal0;		{ For starting data block }
	dm(inptr0)=ax1;
	ax1=1;
	dm(cur_state)=ax1;
	call get_data;			{ Sit here until 512 points collected}
	ax1=^inputreal1;		{ For starting data block }
	dm(inptr1)=ax1;
	ax1=^inputreal2;		{ Buffer part filled, NO FFT }
	dm(dataptr)=ax1;
	do512(2,   inptr0, ^inputreal2, ^inputreal0);
	do512(3,   inptr1, ^inputreal0, ^inputreal1);
	do512(4,   inptr0, ^inputreal1, ^inputreal2);
	do512(5,   inptr1, ^inputreal2, ^inputreal0);
	do512(6,   inptr0, ^inputreal0, ^inputreal1);
	do512(7,   inptr1, ^inputreal1, ^inputreal2);
	do512(8,   inptr0, ^inputreal2, ^inputreal0);

	ay0=b#0000000011111100;		{ Make sure only the right 6 bits }
	ax0=dm(sav_time_tag);
	ar=ax0 and ay0;
	dm(sav_time_tag)=ar;
	dm(status_buf+14)=ar;		{ Mark time tag }
	{ Now, log the sums, clear them and start sending the data. }
	call send12data;		{ It's really 8! }
	ax0=0;
	dm(getn_data)=ax0;		{ A flag }
	jump trig2sec;

	{ This is a highly modified procedure, use when only status bytes
	  are being sent, e.g., with the 4 or  commands for sending
	  dm or pm data to the PC.
	  No data samples are taken, no FFT's occur. In order to not send
	  the 512 bytes of data plus check sum, the following commands
	  should be sent from the PC:
		9 cmd Byte 1, bit 1 =1: Suppress FFT data transmission
		'F'   Bytes 1&2 bit 9=1, bit 11=0: Software trigger
		'F'   Bytes 1&2 bit 14=1: Suppress FFT data collection
		 4 or 5: This command sets the software trigger.
	  The 4 or 5 can be repeated any number of times to get dm
	  or pm. To return to ordinary FFT:
		9 cmd Byte 1, bit 1 =0: Enable FFT data transmission
		'F'   Bytes 1&2 bits 9&11 set per data mode needs.
		'F'   Bytes 1&2 bit 14=0: Enable FFT data collection
	  Note: strange results will occur if 9 command is not used to
	  suppress spectral data transmission, since the data is not being
	  updated, but old data is being sent to the PC.
	}
tnofft:	sr1 = dm(ad_big);		{ Most positive value found }
	dm(status_buf+9) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+10) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(ad_big) = ax0;		{ Zero for next search }

	sr1=dm(agc_gain);
	dm(status_buf+3) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+4) = sr1;		{ Send to pc host }
	sr1 = dm(dac_big);		{ Most positive value found }
	dm(status_buf+11) = sr1;	{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+12) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(dac_big) = ax0;		{ Zero for next search }

	ax0=dm(blk_exponent);		{ Send last blk_exponent from fft }
	dm(status_buf+15)=ax0;
	ax0=dm(PFDATA);			{ Get programable flag reg }
	dm(status_buf+16)=ax0;

	ax0 = 0;
	dm(UART_tx_data_done) = ax0;	{ Reset flag, they can send again }
	jump trig2sec;

{*************************************************************************}
{ send12data Subroutine - Gets result of 12 fft ready to send to pc       }

send12data:
	call log_power;			{ Converts to 8 bit log & dumps int }

	{ Now move to buf0 if buf_select is 1, to buf1 if 0 }
	call move_data;		{ Move data - later use pointers <<<<}

	ax0 = dm(UART_tx_buf_select);	{ Alternate between 0 and 1 }
	ay0 = 1;
	ar = ax0 xor ay0;
	dm(UART_tx_buf_select) = ar;	{ Reverse buffer roles: send and load }
	none = pass ar;			{ Not needed??? }
	ax0 = ^UART_tx_data_buf0;
	if eq jump cgptr;
	ax0 = ^UART_tx_data_buf1;
cgptr:	dm(tx_buf_ptr+1)=ax0;
	sr1 = dm(ad_big);		{ Most positive value found }
	dm(status_buf+9) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+10) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(ad_big) = ax0;		{ Zero for next search }

	ar=dm(serial_error);		{ Look for any errors since last data }
	dm(status_buf+1)=ar;
	dm(serial_error)=ax0;		{ Borrow a zero }
	sr1 = dm(dac_big);		{ Most positive value found }
	dm(status_buf+11) = sr1;	{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+12) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(dac_big) = ax0;		{ Zero for next search }
	ax0=dm(blk_exponent);		{ Send last blk_exponent from fft }
	dm(status_buf+15)=ax0;
	ax0=dm(PFDATA);			{ Get programable flag reg }
	dm(status_buf+16)=ax0;

	{ In non-FM modes, +3 and +4 are agc. In FM, they are Center freq.
	  Instead of jumping over, we write +3 and +4 twice in FM. }
	sr1=dm(agc_gain);
	dm(status_buf+3) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+4) = sr1;		{ Send to pc host }

	{ In FM, send power data, Center freq +3, +4 and Peak Deviation +14 }
	ax0=dm(mode_select);
	ay0=2;				{ See if we are receiving fm }
	none=ax0-ay0;
	if ne jump non_fm;

	{ The average FM freq has been accumulated in 32 bits for 30,779 pts.
	  Divide by 2^15 (scale to 1282.458 counts/kHz) and put in +3 and +4 }
fmdata:	sr0 = dm(fm_ave_l);		{ Least sig delta freq, need 1 bit }
	sr1 = dm(fm_ave_h);		{ Most sig. Use 15 bits of this }
	sr = ashift sr1 by 1 (hi);

	dm(status_buf+3) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+4) = sr1;		{ Send to pc host }
{<<<<<JUST USE SR0 SR1??  <<<<<<<<<< also above, does ashift by 1 pick up sr0?}
	{ Note that our power estimate can exceed 8 bits in pwrh and overflow
	  the log routine.  To prevent this we move everyone down 16 bits
	  (48 dB) and put pwrh in mr1 and pwrm in mr0.  This gives adequate
	  dynamic range (96 dB) for our 15 kHz BW noise and is easy.
	  pwrl is discarded  }
	mr0 = dm(pwrm);  mr1 = dm(pwrh); mr2 = 0;
	call log10_dec;			{ Pwr in dBx*100, out in sr0 }
	sr1 = sr0;			{ dB x 100 in sr0 }
	dm(status_buf+5) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+6) = sr1;		{ Send to pc host }

	{ V2F power level is sent to PC in +7 and +8; Done for FM & non-FM }
non_fm:	ax0=dm(do_v2f);
	ar=tstbit 0 of ax0;		{ See if v2f is running }
	if eq jump nov2f;		{ If eq 0 it is not running }
	sr1=dm(save_db_v2f);		{ In v2f, send dB power lev, +3 & +4} 
	dm(status_buf+7)=sr1;
	sr=lshift sr1 by -8 (HI);
	dm(status_buf+8)=sr1;

 nov2f:	ax0=0;				{ Re-initialize data collection }
	dm(pwrl)=ax0; dm(pwrm)=ax0; dm(pwrh)=ax0;
	dm(fm_ave_h)=ax0;  dm(fm_ave_l)=ax0;
	dm(UART_tx_data_done) = ax0;	{ Reset flag, they can send again }
	rts;

{***************************  GET_DATA  **********************************
 * Waits for 512 data points to be gathered and then returns so that
 * background processing can be done.  Variable get_data_block is 1
 * until 512 data points, then set to 0.
 ************************************************************************}
get_data:
gd1:	ax0=dm(fft_fcn);		{ See if data collection is being
					  cancelled by bit 8 }
	ar=tstbit 8 of ax0;		{ Bit 8 == 1 is stop current collect }
	if gt jump gd2;
	ar=dm(get_data_block);		{ Wait for end of data collection }
	none=pass ar;			{ This loop gets interrupted }
	if gt jump gd1;			{    by input_samples() }
	ax0=1;				{ Fallen through with get_data_block }
	dm(get_data_block)=ax0;		{    =0, so set back to 1 }
	rts;

{ We are now one level down in PC stack and need to stop data collection and
  restore stack.   }
gd2:	ar=clrbit 8 of ax0;		{ Fix up to not return here }
	dm(fft_fcn)=ar;
	ax0=0;
 	dm(data_collect)=ax0;		{ Indicate idle }
	ax0=0XFFFF;
	dm(time_tag)=ax0;		
	call dump_intrgrt;		{ Clear power sums to 0 }
	pop PC;				{ Abandoning get_data() }
	jump again;			{ Wait for new instructions }

{***************************  MOVE_DATA  **********************************
 * Moves data from output of log routine to UART send buffers.
 * Then places 8 bit check sum in 512 position.
 * Should be replaced by pointer movement, but this needs to be
 * integrated into fft module.
 ************************************************************************}
move_data:
	ax0 = dm(UART_tx_buf_select);	{ to buf1 if 0, to buf0 if 1 }
	none = pass ax0;
	i0 = ^UART_tx_data_buf1;
	if eq jump md1;
	i0 = ^UART_tx_data_buf0;
md1:	m0 = 1;    l0 = 0;
	i4 = ^out_log;
	m4 = 1;    l4 = 0;
	ax0 = dm(data_format);		{ See if 16 bit log }
	af=tstbit 0 of ax0;
	if gt jump md2;			{ 1=16bit }
	ay0 = H#00FF;			{ 8 bit mask }
	af = pass 0;			{ Clear check sum }
	cntr = Nov2;
	do md_loop until ce;
	    ax0  = dm(i4, m4);		{ Get data point }
	    ar = ax0 and ay0;		{ Mask to the 8 bits to be transmitted}
	    af = ar + af;		{ Add to check sum }
md_loop:    dm(i0, m0) = ar;
	ax0 = 0;			{ Get checksum to dm,  dm <--no-- af }
	ar = ax0 + af;
	dm(i0, m0) = ar;		{ Append checksum to file }
	rts;

	{ Here to set up Nov2/2 worth of 16 bit data }
md2:	af = pass 0;			{ Clear check sum }
	ay0 = H#00FF;			{ 8 bit mask }
	cntr = Nov4;
	do md2_loop until ce;
	    sr1  = dm(i4, m4);
	    ar = sr1 and ay0;
	    ax0 = ar;
	    af = ax0 + af;		{ Add to check sum }
	    dm(i0, m0) = ar;
	    sr = lshift sr1 by -8 (hi); { Get high order 8 bits into low posn}
	    af = sr1 + af;		{ Add to check sum }
md2_loop:   dm(i0, m0) = sr1;
	ax0 = 0;
	ar = ax0 + af;
	dm(i0, m0) = ar;		{ Append checksum to file }
	rts;

{**********************************************************************
 * xmt_on() - All transmit block functions.  Switches to transmit
 * status data and holds up receive functions.  Others functions are done
 * during interrupt service.
 *
 * During transmit there are still three blocks, but the middle block is
 * changed from the 513 bytes of spectral data to 10 bytes of transmit status.
 * First block:   ESC 'T'
 * Second block:  10 xmit_status_buf
 * Third block:   20 status_buf
 **********************************************************************}
xmt_on:
	{ Check UART_tx_data_done and if 1, set up transmit data block
	  with mike level, power out, ptt, key etc to send in place
	  of receive data.  Change identifier to ESC_CHAR'T' instead of
	  ESC_CHAR'A' }

	{ If measuremnt is triggered, do not send transmit status, regardless
	  of being in transmit.  Prevent log jam at PC.
	  Use this anytime transmit status is not desired. }
	ax0=dm(t_r_control);		{ Bit 1: 0=no status, 1=send status }
	ar=tstbit 1 of ax0;
	if eq jump xo3;			{ Don't send xmit status }

	ax0=dm(UART_tx_data_done);	{ First, wait for rcvr data to leave}
	none=pass ax0;
	if eq jump xmt_on;

	{ Revise data to transmit: }
	ax0=84;				{ 'T' }
	dm(esc_a+1)=ax0;
	ax0=^xmit_status_buf;		{ Transmit Status Data }
	dm(tx_buf_ptr+1)=ax0;		{ Point to it }
	ax0=10;				{ Length of block }
	dm(tx_buf_len+1)=ax0;

	{ While in transmit with status send, loop back to here }
xo1:	sr1 = dm(ad_big);		{ Most positive value found }
	dm(status_buf+9) = sr1;		{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+10) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(ad_big) = ax0;		{ Zero for next search }
	sr1 = dm(dac_big);		{ Most positive value found }
	dm(status_buf+11) = sr1;	{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(status_buf+12) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(dac_big) = ax0;		{ Zero for next search }

	sr1 = dm(del_clip_big);		{ Most positive value found }
	dm(xmit_status_buf+2) = sr1;	{ Send LSB to pc }
	sr=lshift sr1 by -8 (hi);	{ Get the MSBits into lower position }
	dm(xmit_status_buf+3) = sr1;	{ Send to pc host }
	ax0 = 0;
	dm(del_clip_big) = ax0;		{ Zero for next search }

	ax0=dm(PFDATA);			{ Get programable flag reg }
	dm(status_buf+16)=ax0;

	ax0=0;				{ Start sending }
	dm(UART_tx_data_done)=ax0;	{ Tells UART to send 3 Blocks }
xo2:	ax0=dm(UART_tx_data_done);	{ Wait for blocks to be sent to pc }
	none=pass ax0;
	if eq jump xo2;
	ax0=dm(t_r_control);		{ See if back to rcv. Bit 0=1 is xmt }
	af=tstbit 0 of ax0;
	if gt jump xo1;			{ Continue to wait for receive }

	{ Revise data to receive: }
	ax0=65;				{ 'A' }
	dm(esc_a+1)=ax0;
	ax0=^status_buf;		{ Transmit Status Data }
	dm(tx_buf_ptr+1)=ax0;		{ Point to it }
	ax0=513;			{ Length of block }
	dm(tx_buf_len+1)=ax0;
	{Change to 'jump xmt_on', then add 5 msec delay here;
         Need mute bits?
	 Then 'jump again'
	}
	rts;				{ Let reception begin again }
	
	{ Short loop for the case of 'no transmit status data': }
xo3:	ax0=dm(t_r_control);		{ See if back to rcv. Bit 0=1 is xmt }
	af=tstbit 0 of ax0;
	if gt jump xo3;			{ Continue to wait for receive }
	rts;				{ Back to receive }

{***   End Background routines,  Start ISR's    ************************}

{***********************************************************************
 *                 TIMER Interrupt Handler                             *
 *  TIMER put us here 2 times per second. Used for triggered mode to   *
 *  start the collection of a new 2 second data series.                *
 *  >>>>>>>>>>>>>  NOTE - not used at this time.  RSL 28 May 00  <<<<  *
 ***********************************************************************}
#if 0
timer_int:
	dis ints;
	dm(temp2)=ar;			{ Don't use temp2, temp3 elsewhere }
	dm(temp3)=ay0;
	ay0=dm(timer4);			{ Count four 1/2 sec periods }
	ar=ay0-1;
	if eq jump ti1;			{ We are at 2 second mark }
	dm(timer4)=ar;			{ Not there, setup for next time }
	ar=dm(temp2);
	ay0=dm(temp3);
	ena ints;
	rti;
ti1:	ay0=4;				{ End of 2 sec gets us here }
	dm(timer4)=ay0;			{ Reset software counter }
	{ Put things here that need to happen after 2 sec }
	ar=dm(temp2);			{ Return from int }
	ay0=dm(temp3);
	ena ints;
	rti;
#endif

{***********************************************************************
 *                 SPORT0 Interrupt Handler                            *
 *  SPORT0 inputs from the CODEC put us here 48000 times per second.   *
 *  Routines here are those that need this periodic service.           *
 ***********************************************************************}
input_samples:
        ena sec_reg;                    { use shadow register bank }
	set FL1;			{ Monitor point }
        dm(i0_sav)=i0; dm(m0_sav)=m0; dm(l0_sav)=l0; { Free up DAG 0 & 4 }
        dm(i4_sav)=i4; dm(m4_sav)=m4; dm(l4_sav)=l4;

	{ The following is not used by DSP-10 except delay.
	  This supports experimental binaural reception.  RSL }
	{ Now narrow or wide processing based on ssb_select as follows:
	Bit 0	0=LSB		1=USB		(narrow-band)
	Bit 1	0=monaural	1=binaural
	Bit 2	0=LSB/USB	1=I-Q source
	Bit 3
	Bit 4	0=Reverse L/R	1=Normal L/R	(not implemented yet )
	Bit 7   0=Narrow Band	1=Wideband processing
	}
	ax0 = dm(ssb_select);
	af = tstbit 7 of ax0;
	if gt jump wis0;		{ Wideband processing }

	mx0=dm(rx_buf+1);		{ Get i-f sample from SPORT0 (codec) }
	dm(save_if)=mx0;
	ar = mx0;
	ar = abs ar;			{ Look for biggest input }
	ay1 = dm(ad_big);
	none = ar - ay1;
	if lt jump is1;			{ Not as big as previous biggest }
	dm(ad_big) = ar;		{ Save for next time }
is1:
	ax0 = dm(t_r_control);		{ Bit 0 = 1 is xmt }
	af=tstbit 0 of ax0;
	if gt jump xmt_isr;		{ Do xmit functions }

	ax0=dm(mode_select);
	ay0=2;				{ See if we are receiving fm }
	none=ax0-ay0;
	if eq jump fm_rcvr;

	my0 = dm(r_if_gain);
	mr = mx0 * my0 (SS);
	my0 = mr1;			{ Gain corrected i-f signal }

	ax1 = dm(if_dphase);		{ Phase increment for lo }
	ay1 = dm(if_phase);		{ Last phase }
	ar = ax1 + ay1;			{ New phase }
	ax0 = ar;
	dm(if_phase) = ar;		{ For next time }
	call sin;			{ Phase in ax0, Sin returned in AR }
	mr=ar*my0(SS);			{ I mixer }

	i0=dm(fir1i_i0_sav);  m0=1;  l0=%idata;
	dm(i0,m0) = mr1;		{ Store sample in fir1 delay line }
        dm(fir1i_i0_sav)=i0;  		{ i0 for next call of do_fir1i }

	ax0 = dm(if_phase);
	ay0 = 16384;  ar = ax0+ay0;	{ 90 degrees for quadrature lo }
	ax0 = ar;  call sin;		{ Cos lo signal, sin() preserves my0 }
	mr=ar*my0(SS);			{ Sig still in my0, Q mixer }
	i0=dm(fir1q_i0_sav);  m0=1;  l0=%qdata;
	dm(i0,m0) = mr1;		{ Store sample in fir1q delay line }
        dm(fir1q_i0_sav)=i0;		{ i0 for next call of do_fir1q }

	ar=dm(ssb_select);
	ay1=2;
	ar=ar-ay1;
	if eq jump binaural;

{ Note a rate break occurs here.  Up to here is 48 kHz.  Blocks1 to Block5
 run at 9.6 kHz and do audio processing.  The next call, interp_filter
 is again at a 48 kHz rate and gets it's input from
 the fir2 delay line that was loaded at 9.6 kHz along with 4
 zero data points between every signal data point.    }

is2:	call interp_filter;		{ LPF for interpolation, out in mr1 }
	{ If using V2F tone generator, the normal AF gain is not used, as the
	  level was set by the sidetone level, in block 4 }
	ax0=dm(do_v2f);
	sr1=mr1;			{ Ready, if jump to is2_v }
	ar=tstbit 0 of ax0;
	if gt jump is2_v;		{ Jump over AF Gain }
	my0 = dm(af_gain);
	mr = mr1 * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);		{ Gain adjusted audio in SR1 }
is2_v:
	{ The audio signal to this point has been kept at 15 bits to prevent
	  clipping in the filters, which could be very bad.
	  Now, raise the signal up one bit to allow the full DAC range to
	  be available. }
	ena AR_SAT;			{ Last-gasp clipper }
	ay1 = sr1;
	ar = sr1 + ay1;			{ Last needed x2 }
	dis AR_SAT;
	sr1 = ar;

	{ Note: tx_buf+1 happens to be same for both 1847 & AIC23 Codecs.  }
	dm(tx_buf+1) = sr1;		{ Audio output }

 	ar=dm(out_steer);
	none=pass ar;
	if gt jump is2a;

	{ Delay of right channel by amount delay_right.  If set to 1 there is
	  no delay. Undelayed data in sr1.  }
	i0=dm(del_i0_sav); l0=dm(delay_right);
	m0=0;
	mr1=dm(i0, m0);			{ Get delayed data to mr1 }
	m0=1;
	dm(i0, m0)=sr1;			{ Enter newest data to line }
	dm(del_i0_sav)=i0;

#if CODEC_1847
	dm(tx_buf+2) = mr1;  		{   L & R }
#endif
#if CODEC_AIC23
	dm(tx_buf+0) = mr1;
#endif

is2a:
	ar = abs mr1;			{ Hunt for largest magnitude output }
	ay1 = dm(dac_big);		{  to be sent to pc }
	none = ar - ay1;
	if lt jump is3;			{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }

is3:	call uart_rcvr;			{ See if character is coming in }

	ay0 = 1;
	ar = dm(block_count);		{ Ranges from 0 to 4 }
	ar = ar + ay0;			{ Update to next block }
	dm(block_count) = ar;		{ Save for next interrupt, now 1 to 5 }
	ar = ar - ay0;			{ Test for current block }
	if eq jump block1;
	ar = ar - ay0;
	if eq jump block2;
	ar = ar - ay0;
	if eq jump block3;
	ar = ar - ay0;
	if eq jump block4;
	jump block5;			{ This block will reset count to 0 }

end_blocks:
	i0=dm(i0_sav); m0=dm(m0_sav); l0=dm(l0_sav); { Restore DAG 0 & 4 }
        i4=dm(i4_sav); m4=dm(m4_sav); l4=dm(l4_sav);
	reset FL1;
        dis sec_reg;
        rti;

{ For binaural  reception, we do no decimation for filtering.  Still need a
  1/5 processing rate for UART transmission.  }
binaural:
	call do_fir1i;			{ LPF for I, out in mr1 }
	my0 = dm(af_gain);
	mr = mr1 * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);
	{ Note: tx_buf+1 happens to be the same for 1847 & AIC23 }
	dm(tx_buf+1) = sr1;		{ Audio output }

	call do_fir1q;			{ LPF for Q, out in mr1 }
	dm(audsav) = mr1;		{ To send to the FFT }
	my0 = dm(af_gain);
	mr = mr1 * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);

#if CODEC_1847
 	dm(tx_buf+2) = sr1; 		{ Audio Output }
#endif
#if CODEC_AIC23
	dm(tx_buf+0) = sr1;
#endif

	ar = abs sr1;			{ Hunt for largest magnitude output }
	ay1 = dm(dac_big);		{  to be sent to pc, Q only }
	none = ar - ay1;
	if lt jump bin1;		{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }

bin1:	call uart_rcvr;			{ See if character is coming in }

	ay0 = 1;
	ar = dm(block_count);		{ Ranges from 0 to 4 }
	ar = ar + ay0;			{ Update to next block }
	dm(block_count) = ar;		{ Save for next interrupt, now 1 to 5 }
	ar = ar - ay0;			{ Test for current block }
	if eq jump end_blocks;          {if eq jump bin_bl1; }
	ar = ar - ay0;
	if eq jump end_blocks;		{ if eq jump bin_bl2; }
	ar = ar - ay0;
	if eq jump bin_bl3;
	ar = ar - ay0;
	if eq jump bin_bl4;
	jump bin_bl5;
bin_bl3:
	ax0=0X0B;
	dm(status_buf+8)=ax0;
	ar=dm(data_collect);		{ Collecting or not? }
	ar=pass ar;			{ Set flags }
	if eq jump end_blocks;		{ Don't collect data }
	ay0=dm(fft_buff_count);		{ Put in inputreal[]  }
	ar=ay0-1;
	dm(fft_buff_count)=ar;
	if lt jump b_bl3c;		{ if buffer full }
b_bl3a:	i0=dm(inptr0);  m0=1;  l0=N;
	ar=dm(audsav);			{ Get new data point }
	dm(i0, m0)=ar;			{ Data to fft input }
	dm(inptr0)=i0;
	i0=dm(inptr1);
	dm(i0, m0)=ar;			{ Second one for overlapping window }
	dm(inptr1)=i0;
	jump end_blocks;

b_bl3c:	ax0=0;
	dm(get_data_block)=ax0;		{ Here since buf full, flag as such }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }
	jump b_bl3a;

{ Routines for fm reception running at 48 kHz rate. Calculates the incoming
  signal phase relative to a center frequency carrier.  This
  phase subtracted from the previous measurement.
  This difference is then low-pass filtered as the demodulated result.
  Latest incoming sample in mx0.   }
fm_rcvr:
	call fir10hp;			{ Input in mx0, output in mr1 }
	dm(temp1)=mr1;

	my0=mr1;
	mr=mr1*my0 (SS);
	ax0=dm(pwrl); ay0=mr0; ar=ax0+ay0;	{ Accumulate power (48 bit) }
	dm(pwrl)=ar;
	ax0=dm(pwrm); ay0=mr1; ar=ax0+ay0+c;
	dm(pwrm)=ar;
	ax0=dm(pwrh); ay0=0;   ar=ax0+ay0+c;
	dm(pwrh)=ar;

	my0=dm(temp1);
	ax1 = dm(if_dphase);		{ Phase increment for lo }
	ay1 = dm(if_phase);		{ Last phase }
	ar = ax1 + ay1;			{ New phase }
	ax0 = ar;
	dm(if_phase) = ar;		{ For next time }
	call sin;			{ Phase in ax0, Sin returned in AR }
	mr=ar*my0(SS);			{ I mixer }
	call fir6lp1;			{ 6 KHz, In & out mr1, preserves my0}
	si=mr1;				{ Save the in-phase component }
	ax0 = dm(if_phase);
	ay0 = 16384;  ar = ax0+ay0;	{ 90 degrees for quadrature lo }
	ax0 = ar;  call sin;		{ Cos lo signal, sin() preserves my0 }
	mr=ar*my0(SS);			{ Sig still in my0, Q mixer }
	call fir6lp2;
	ax1 = si;
	ar  = mr1;
	call arctan;			{ Returns arctan(ar/ax1) in ar }
	{ -1.0 in 1.15 (H#8000) is -180 degrees and .9999 (H#7FFF) is
	  180 degrees.  When we take the phase difference in 2's complement
	  arithmetic the answer will be in (-180, 180) without further
	  operations.  }
	ay0=dm(last_ph);		{ Get previous measurement }
	dm(last_ph)=ar;			{ Save for next time }
	ar=ar-ay0;			{ Demodulated signal ready for LPF }
	i0=dm(fir1i_i0_sav);  m0=1;  l0=%idata;
	dm(i0,m0) = ar;			{ Store sample in fir1i delay line }
        dm(fir1i_i0_sav)=i0;  		{ i0 for next call of do_fir1i }

	{ Discriminator sensitivity is 65536/48000 = 1.365333 LSB/Hz }
	{ Test Points for FM. Add in an offset of 13653 to simplify averaging }
	ay0=13653;			{ Offset 10 kHz = 13653 }
	ar=ar+ay0;
	ay0=dm(fm_ave_l); ar=ar+ay0;	{ Add in new discriminator output }
	dm(fm_ave_l)=ar;
	ax0=dm(fm_ave_h); ay0=0; ar=ax0+ay0+c;  { Accumulate carries }
	dm(fm_ave_h)=ar;
	jump is2;

{***************************************************************************
 * Receive Block subroutines - Called by the 48KHz interrupt routine
 * one after another at a 9.6 KHz rate. Max of xxx cycles per block.
 * Note that do_fir1i and do_fir1q LPF are calculated once for every 5 data
 * points that are entered, i.e., we only calculate the outputs
 * that we will use.  This decimation works since the 9.6kHz sample
 * rate satisfies Nyquist for the cutoff frequency of the LPF.
 ***************************************************************************}
block1: ar=dm(save_if);			{ A/D sample }
	dm(v_v2f1)=ar;			{ Save data for v2f }
	call do_fir1i;			{ Decimation filter for I }
	dm(isave) = mr1;

	ax0=dm(mode_select);
	ay0=2;				{ See if we are receiving fm }
	none=ax0-ay0;
	if eq jump bl1_fm;
	call do_fir1q;			{ Decimation filter for Q }
	dm(qsave) = mr1;
bl1_fm:	add_point;			{ Put zero into output line }
	jump end_blocks;

block2: ar=dm(save_if);			{ A/D sample }
	dm(v_v2f2)=ar;			{ Save data for v2f }
	ax0 = dm(isave);		{ I signal for delay }
	ax1=dm(mode_select);
	ay1=2;				{ See if we are receiving fm }
	none=ax1-ay1;
	if eq jump bl2fm;
	ay0 = dm(qsave);		{ Q signal for Hilbert }
	call do_hilbert2;		{ 90 deg, returns I=ax1, Q=mr1 }
	ay1 = mr1;
	ar = ax1 - ay1;			{ - = lsb }
	ax0=dm(ssb_select);		{ 0=lsb, 1=usb }
	none=pass ax0;
	if eq jump bl2a;
	ar = ax1 + ay1;			{ + = usb }
bl2a:	dm(audsav) = ar;		{ Sav for next block }

	{ Calculate the power at audio. This is presently used when the
	  spectral display is off-screen and the power would otherwise
	  be inaccurate. 14 instructions. }
	my0=ar;
	mr=ar*my0 (SS);			{ Power }
	ax0=dm(pwrl); ay0=mr0; ar=ax0+ay0;	{ Accumulate power (48 bit) }
	dm(pwrl)=ar;
	ax0=dm(pwrm); ay0=mr1; ar=ax0+ay0+c;
	dm(pwrm)=ar;
	ax0=dm(pwrh); ay0=0;   ar=ax0+ay0+c;
	dm(pwrh)=ar;			{ Stored same as FM power estimate }

	add_point;
	jump bl2_ser;

bl2fm:	dm(audsav) = ax0;		{ Save audio signal for next block }
	add_point;			{ Zero for interp filter }

xbl2:					{ Entry point for shared transmit }
bl2_ser:
	ay0=dm(UART_tx_data_done);	{ See if we are xmtg thru serial port }
	none = pass ay0;
	if gt jump end_blocks;
	ay0=dm(UART_tx_ch_count);	{ New blk if down to 0 }
	ar=pass ay0;
	if gt jump bl5c;		{ No new data block yet }

	ay0=dm(UART_tx_data_block);	{ Move to next data block }
	ar=ay0+1;
	dm(UART_tx_data_block)=ar;	{   and save it }
	ay0=ar;
	ay1=DATA_BLOCKS;
	ar=ar-ay1;			{ See if up to full number of blocks }
	if ne jump bl5;
	ay1=1;				{ Time to start new data blocks }
	dm(UART_tx_data_done) = ay1;	{ And flag it to the task managers }
	ay1=-1;				{ Reset data block counter }
	dm(UART_tx_data_block) = ay1;
	{ Now exit with data_done flag set, data block=-1 and
          ch_count=0. When data_done is reset to 0 we will increment
          data_block to 0 and start all over }
	jump end_blocks;

	{ Arrive at bl5 with ay0=block number }
bl5:					{ Table lookup of data blk parameters}
	ax0=^tx_buf_ptr;		{ ay0 is now index, 0...DATA_BLOCKS-1 }
	ar=ax0+ay0;			{ Turn it into an address of pointer }
	i0=ar;  m0=0;  l0=0;
	ax1=dm(i0, m0);
	dm(UART_tx_ptr)=ax1;		{ Set current pointer }
	ax0=^tx_buf_len;
	ar=ax0+ay0;
	i0=ar;
	ax1=dm(i0, m0);
	dm(UART_tx_ch_count)=ax1;	{ Set buffer length }
	none = pass ax1;
	if eq jump end_blocks;		{ Feature not used, does it work?? }
	ax0=^tx_buf_word_len;
	ar=ax0+ay0;
	i0=ar;
	ax1=dm(i0, m0);
	dm(UART_tx_word_len)=ax1;	{ Set byte/word--not currently used }
	i0=dm(UART_tx_ptr);		{ Set pointer to first data byte }
	ax0=dm(i0, m0);			{ Get byte (word) }
	dm(UART_tx_cur_data)=ax0;	{ Save new data }
	ay0=ESC_CH;			{ Check if first ch in data block }
	ar=ax0-ay0;			{   is ESC_CH.  If so, flag for now }
	if ne jump bl5a;		{   as data by setting tx_esc=1 }
	ax0=1;
	dm(UART_tx_esc)=ax0;
bl5a:
	ax0=dm(UART_tx_ch_count);	{ If new block, check if esc block }
	ar=pass ax0;
	if eq jump end_blocks;
	ay0=2;				{ Now see if  data block length }
	ar=ax0-ay0;			{   is 2.  If so we will ignore }
	if ne jump bl5b;		{   the leading ESC_CH by setting }
	ax0=0;				{   tx_esc=0.  Longer blocks will }
	dm(UART_tx_esc)=ax0;		{   be left with a double esc ch }
bl5b:	ax0=10;
	dm(UART_tx_cur_bit)=ax0;
bl5c:	call out_block_byte;
	jump end_blocks;

block3:	{ dm(audsav) has the audio signal sample. If the FFT is running at
	  4.8 or 2.4 kHz sample rate, it needs to be LP filtered and then
	  every 2nd or 4th sample is put into FFT input.
	}
	ar=dm(save_if);			{ A/D sample }
	dm(v_v2f3)=ar;			{ Save data for v2f }
	ax0=dm(fft_fcn);
	ay0=0x0003;			{ Check bit 0->2400 Hz, bit 1 4800 }
	none=ax0 and ay0;		{ Set flags }
	if eq jump bl3aa;		{ They are both 0 }
	ar=tstbit 0 of ax0;		{ See if 2400 }
	if eq jump bl3ab;		{ Not 2400, must be 4800 }

	{ Here to process 1200 Hz BW (2400  sample rate) }
	mr1=dm(audsav);			{ Input to Nyquist LPF }
	call fir1200;			{ mr1 in, mr1 out }
	ar=dm(divide_count);		{ 4,3,2,1 cycle }
	ar=ar-1;			{ Count down }
	if eq jump bl3ac_0;		{ Time to take sample (=0) }
	dm(divide_count)=ar;		{ No sample, just return count }
	jump bl3ad;			{ And go on with processing }

bl3ac_0:{ Put sample into FFT's and reset divide_count }
	ax0=4;
	dm(divide_count)=ax0;		{ Reset count to 4 }
	ay0=dm(fft_buff_count);		{ Try to put in inputreal[]  }
	ar=ay0-1;			{ See if there is room }
	dm(fft_buff_count)=ar;
	if lt jump bl3c1_0;		{ if buffer full }
bl3ac1_0: i0=dm(inptr0);  m0=1;  l0=N;
	dm(i0, m0)=mr1;			{ Data from LPFIR to fft input }
	dm(inptr0)=i0;
	i0=dm(inptr1);
	dm(i0, m0)=mr1;			{ Second one for overlapping window }
	dm(inptr1)=i0;
	jump bl3ad;	

bl3ab:	{ Here to process 2400 Hz BW (4800 sample rate) FFT input }
	mr1=dm(audsav);			{ Input to Nyquist LPF }
	call fir2400;			{ mr1 in, mr1 out }
	ar=dm(divide_count);		{ 2, 1 cycle }
	ar=ar-1;			{ Count down }
	if eq jump bl3ac;		{ Time to take sample (=0) }
	dm(divide_count)=ar;		{ No sample, just return count }
	jump bl3ad;			{ And go on with processing }

bl3ac:	{ Put sample into FFT's and reset divide_count }
	ax0=2;
	dm(divide_count)=ax0;		{ Reset count to 2 }
	ay0=dm(fft_buff_count);		{ Try to put in inputreal[]  }
	ar=ay0-1;			{ See if there is room }
	dm(fft_buff_count)=ar;
	if lt jump bl3c1;		{ if buffer full }
bl3ac1:	i0=dm(inptr0);  m0=1;  l0=N;
	dm(i0, m0)=mr1;			{ Data from LPFIR to fft input }
	dm(inptr0)=i0;
	i0=dm(inptr1);
	dm(i0, m0)=mr1;			{ Second one for overlapping window }
	dm(inptr1)=i0;
	jump bl3ad;	

bl3aa:	{ For 9.6 sample rate, put data into FFT inputs }
	ay0=dm(fft_buff_count);		{ Try to put in inputreal[]  }
	ar=ay0-1;			{ Count inputs and check for room? }
	dm(fft_buff_count)=ar;
	if lt jump bl3c;		{ Buffer is full }
bl3a:	i0=dm(inptr0);  m0=1;  l0=N;
	ar=dm(audsav);			{ Get new data point }
	dm(i0, m0)=ar;			{ Data to fft input }
	dm(inptr0)=i0;
	i0=dm(inptr1);
	dm(i0, m0)=ar;			{ Second one for overlapping window }
	dm(inptr1)=i0;

bl3ad:	{ Process following entry of data for FFT's }
	ar=dm(audio_fir_on);		{ See if audio_fir is being used }
	none=pass ar;
	ar=dm(audsav);			{ Get new data point again }
	mr1=dm(audsav);			{ In case there is no FIR }
{ Doesn't changing FIR in and out to mr1 stop all these problems? }
	if ne call audio_fir;		{ ar in, mr1 out }

	ax1=dm(mode_select);
	ay1=2;				{ Are we in fm mode? }
	none=ax1-ay1;
	if ne jump bl3b;		{ Not in fm mode }
	ax1=dm(squelch_noise);		{ Noise from fft, bigger for weaker }
	ay1=dm(squelch);
	none=ax1-ay1;			{ Compare }
	if lt jump bl3b;		{ Squelch is open }
	mr1=0;				{ No audio, squelch closed }
bl3b:	ar=dm(agc_decay);		{ Apply AGC if On }
	none=pass ar;
	if eq jump bl3b8;		{ AGC not being used }
{ AGC -  agc_gain attempts to keep the 'max' envelope at AGC_SET_POINT.
  agc_gain can range from 2^8-1=255 down to 1.0 and is stored as an
  unsigned number in 8.8 representation with bit values of:
                   2^7, 2^6, 2^5, ... 2^-8
  The max gain is 255 to keep from having excessive background noise,
  and the minimum is 1 or else A/D overload occurs before full audio.
  This gives 48 dB of AGC range, adequate for many purposes.
  An additional 36 dB is available from manual RF gain for 84 dB total. }

	my0=dm(agc_gain);		{ 8.8 unsigned format gain }
	mr=mr1*my0 (SU);
	none=pass mr1;			{ Neg if sig was neg }
	if lt jump bl3b0;
	ay0=0XFF00;			{ Top 8 bits should be 0 }
	none=mr1 and ay0;		{ See if they are }
	if eq jump bl3b00;		{ No overload }
	mr1=0X00FF;			{ Max positive level }
	mr0=0XFF00;
	jump bl3b00;
bl3b0:	ay0=0XFF00;
	ar=mr1 and ay0;
	ay0=0XFF00;
	none=ar-ay0;
	if eq jump bl3b00;
	mr1=0XFF00;
	mr0=0X0000;
bl3b00:	sr=ashift mr1 by 7 (hi);
	sr=sr OR lshift mr0 by 7 (lo);

	dm(temp1)=sr1;			{ AGC adjusted audio level }

	{ Update the envelope by the current audio signal, in sr1 & dm(temp1) }
	ar=abs sr1;
	sr=ashift ar by -1 (hi);	{ sr1 is now abs of sig / 2 }
	ay0=dm(audio_env);
	none=sr1-ay0;			{ See if growth or decay }
	if le jump bl3b1;		{ Jump if current sig > env, ie decay }
	ar=sr1+ay0;			{ Half new, half old }
	sr=ashift ar by -1 (hi);	{ Here's the half }
	dm(audio_env)=sr1;
	jump bl3b2;			{ Audio sig still in mr1 }

bl3b1:	{ Here to decay envelope since current sig is bigger than old env }
	my0=655;			{ 0.02 }
	mr=sr1*my0 (SS);		{ Times current abs val of sig }
	my0=32112;			{ 0.98 }
	mx0=dm(audio_env);
	mr=mr + mx0*my0 (SS);		{ Slight decay }
	dm(audio_env)=mr1;

bl3b2:	{ Now take log of envelope to keep agc gain uniform with sig level }
	mr1=dm(audio_env);
	none=pass mr1;			{ Set result to zero if input  }
	if gt jump rr0;			{    is le zero }
	ar=0;
	jump rr1;
rr0:	se=exp sr1 (hi);		{ Log is linear approx between each }
	ax0=se;				{   octave of range.  Continuous }
	ay0=15;				{   function but discontinuous first }
	ar=ax0+ay0;			{   derivative }
	my0=1024;
	mr=ar*my0 (SS);
	sr=norm sr1 (hi);
	{ sr1 is a value between 0100000...B and 0111111...B and needs to have
	  0100000...B subtracted off and then the whole thing doubled }
	ay0=0X4000;
	ar=sr1-ay0;
	{ After doubling we need to move the mantissa over -4.  Including the
	  doubling we will move it -3:   }
	sr=lshift ar by -3 (hi);
	ay0=sr1;
	ar=mr0+ay0;
rr1:	{ Time to update agc_gain }
	ay1=ar;				{ Log of audio_env }
	ax1=AGC_SET_POINT;
	ar=ax1-ay1;
	if le jump bl3b3;		{ Gain too high, go quickly and fix }
	{ Here because gain is too low---increase slowly }
	my1=dm(agc_decay);		{ decay constant, kd }
	mr=ar*my1 (SS);
	none=pass mr1;			{ Don't allow zero }
	if gt jump bl3b2a;
	mr1=1;
bl3b2a:	ay1=dm(agc_gain);		{ Old agc_gain, go }
	ar=mr1+ay1;			{ New agc_gain: g=go+kd*d }
	{ Checks on agc_gain hi.  Here we first divide the unsigned gain
	  by two to prevent sign problems during subtraction.  The actual
	  agc_gain is left as 16 bit unsigned number for resolution }
	dm(agc_gain)=ar;		{ ar is proposed new agc_gain }
	sr=lshift ar by -1 (hi);	{ Get some room (unsigned num) }
	ay0=AGC_GAIN_MAX_D2;		{ See if too high }
	none=sr1-ay0;
	if lt jump bl3b5;		{ Not too high or too low }
	ar=AGC_GAIN_MAX;		{ Reduce to max gain }
	dm(agc_gain)=ar;
	jump bl3b5;

bl3b3:	{ Here because gain is too high---decrease quickly (fast attack) }
	my1=AGC_GROWTH;			{ growth constant, kg }
	mr=ar*my1 (SS);
	ay1=dm(agc_gain);		{ Old agc_gain, go }
	ar=mr1+ay1;			{ New agc_gain: g=go+kg*d, d<=0 }
bl3b4:	{ Checks on agc_gain low.    }
	dm(agc_gain)=ar;		{ ar is proposed new agc_gain }
	sr=lshift ar by -1 (hi);	{ Get some room (unsigned num) }
	ay0=AGC_GAIN_MIN_D2;		{ See if gain is too low }
	none=sr1-ay0;
	if ge jump bl3b5;		{ OK go use new gain }
	ar=AGC_GAIN_MIN;		{ Increase to min gain }
	dm(agc_gain)=ar;
bl3b5:	mr1=dm(temp1);			{ Reenter here after agc gain set }
	{ End AGC processing }

bl3b8:	dm(save3to4)=mr1;		{ Re-enter here if audio in mr1 }
	add_point;			{ Add 0 to output line }
	jump end_blocks;

	{ Reset processing for FFT input, 9600 rate }
bl3c:	ax0=0;
	dm(get_data_block)=ax0;		{ Here since buf full, flag as such }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }
	jump bl3a;

	{ Reset processing for FFT input, 2400 rate }
bl3c1_0:  ax0=0;
	dm(get_data_block)=ax0;		{ Here since buf full, flag as such }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }
	jump bl3ac1_0;

	{ Reset processing for FFT input, 4800 rate }
bl3c1:	ax0=0;
	dm(get_data_block)=ax0;		{ Here since buf full, flag as such }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }
	jump bl3ac1;

block4: ar=dm(save_if);			{ A/D sample }
	dm(v_v2f4)=ar;			{ Save data for v2f }

	{	add_point; }

bin_bl4:
	{ Following does any needed programming of shift-registers/synth }
	ax0=dm(sr_count);		{ See if a bit data left in sr_data }
	none=pass ax0;
	if eq jump bl4e;		{ Nothing to shift }
	af=tstbit 1 of ax0;		{ Last 2 bits indicate operation }
	if eq jump bl4b;
	{ Here because bit pattern 11, i.e., Set data according to bit 15
	  of sr_data; first clear bit 1 of sr_count to skip a count }
	ar=clrbit 1 of ax0;
	dm(sr_count)=ar;
	ax1=dm(sr_data);
	ar=tstbit 15 of ax1;
	sr0=ax1;
	sr=lshift sr0 by 1 (lo);	{ Move bits over, status unaffected }
	dm(sr_data)=sr0;		{ Save shifted data }
	if eq jump bl4a;
	set FL0;			{ Data line to 1 }
	jump bl4e;
bl4a:	reset FL0;			{ Data line to 0 }
	jump bl4e;
bl4b:	{ Here because bit 1 was 0, i.e., do a clock pulse }
	af=tstbit 0 of ax0;		{ 1 at rise of clock, 0 at fall}
	if eq jump bl4c;
	{ Here because bit 1 was 0 and bit 0 was 1:  Clock rise }
	set fl2;			{ Clock line }
	jump bl4d;
bl4c:	reset fl2;			{ Clock fall }
bl4d:	ar=ax0-1;
	dm(sr_count)=ar;
bl4e:
	{ End hardware programming }

	mr1=dm(save3to4);		{ Audio left from block 3 }
	ar=dm(audio_LMS_on);		{ See if LMS is being used }
	none=pass ar;
	if le jump bl4f;
	call do_LMS;			{ De-noise, mr1 in, mr1 out }
	jump bl4g;			{ Never auto-notch also }
bl4f:	if lt call do_autonotch;	{ Auto-Notch, mr1 in and out }

bl4g:	ax0=dm(mode_select);
	ay0=2;				{ See if we are receiving fm }
	none=ax0-ay0;
	if ne jump bl4ff;
	{ Need  de-emphasis for NBFM.  Do this with mini-IIR filter: }
	{ 3244,29524 is 1000 usec, but this just sounds bassey. Arbitrarily go
 	  with 7756,25012, with tau=1/(2*pi*500Hz) }
	my0 = 7756;			{ K = 0.2367*32768, 9.6 KHz fs }
	mr  = mr1*my0 (ss);		{ K*x }
	mx0 = dm(save_out);		{ Get last output }
	my0 = 25012;			{ 1.0-K }
	mr = mr+mx0*my0 (ss);		{ y(new)=K*x+(1-K)*y(last) in MR1 }
	dm(save_out) = mr1;
bl4ff:
	{ If using voltage-to-frequency feature to indicate power:  }
	ax0=dm(do_v2f);			{ Control word }
	ar=tstbit 0 of ax0;
	if eq jump bl4fff;		{ Jump if no V2F }

	{ Here for V2F. dm(do_v2f) controls the data source  }
	ay0=0X82;			{ Bit 1=1 sigs, Bit 7=1 Peak }
	ar=ax0 and ay0;
	none=ar-ay0;			{ Both 1 -> signal and peak  }
	if eq jump b4v2f2;		{ Add in signal peak }
	ay0=0X02;			{ See if signal average }
	none=ar-ay0;
	if eq jump b4v2f3;		{ Add in signal average }

	{ Bit 1==0 falls through, for either state of bit 7 }
	ax0=0;				{ Zero power sum over blocks }
	dm(pwr_v2f_l)=ax0;
	dm(pwr_v2f_h)=ax0;
	mx1=dm(v_v2f1);			{ Data from block 1 }
	call pwr_v2f;			{ Filter DC, add to power average }

	mx1=dm(v_v2f2);			{ Data from block 2 }
	call pwr_v2f;			{ Filter DC, add to power average }

	mx1=dm(v_v2f3);			{ Data from block 3 }
	call pwr_v2f;			{ Filter DC, add to power average }

	mx1=dm(v_v2f4);			{ Data from block 4 }
	call pwr_v2f;			{ Filter DC, add to power average }

	mx1=dm(v_v2f5);			{ Data from block 5 }
	call pwr_v2f;			{ Filter DC, add to power average }
	{ The power is now 5x too big, but only relative power is used,
	  so this is of no significance. Current power measurement
	  is in pwr_v2f_l and _h.  }

	{ To make the averaging filter stable with a 32-bit result, do
	  log first. Otherwise, when the signal level is small and omt_v2f
	  is small (long time constant) the product needs more than 32 bits
	  (32bits * 32 bits is 64 bits) and underflows. Logging first prevents
	  having small signal values, but also is not a true power ave. }
	  
	{ Convert to log, add in offset, multiply by scale }
        mr0=dm(pwr_v2f_l);		{ Get power mantissa, lo }
        mr1=dm(pwr_v2f_h);		{   and high. }
	{ Entry point for data collected from FFT, for signals }
b4v2f1:	mr2=0;				{ mr2 is always 0 (high 8 bits) }
        call log10;			{ Log10 in 4.12 in sr1 }
	{ Dynamic range can be a problem when we subtract later. But, we have
	  about 160 dB here to play with, so use some to offset the dB's.
	  This also avoids the region around 0 which is problematic for the
	  running average calculation. }
	ay0=15000;
	ar=sr1+ay0;

	{ The running average power is a 32 bit value that is updated 9600
	  times per second to set the tone frequency. This is done with an
	  "RC" filter (see EMRFD p10.13) }
	mx0=dm(rng_pave_l); mx1=dm(rng_pave_h);  { Current power average }
	my0=dm(t_v2f_l);    my1=dm(t_v2f_h);     { Scaling coeff RC filter }
	call mult32;
	ax0=mr0;  ax1=mr1;		{ Save for a moment }
	mx0=0;  mx1=ar;			{ Log of last measurement }	
	my0=dm(omt_v2f_l);    my1=dm(omt_v2f_h);  { The coefficient }
	call mult32;			{ (SS), 32-bit, result in mr0 and mr1 }

	{ Add diminished sample (mr0, mr1) to decayed running ave (ax0, ax1)}
	ay0=mr0; ar=ax0+ay0;		{ Low 16 bits }
	dm(rng_pave_l)=ar;		{ Save for next time }
	ay1=mr1; ar=ax1+ay1+c;		{ High 16 bits }
	dm(rng_pave_h)=ar;

	dm(save_db_v2f)=ar;		{ Send to PC to allow setting offset }
        ay0 = dm(offset_v2f);		{ Center freq range of v2f }
        ar = ar-ay0;			{ Diff in dB from offset, can be +/- }
 
	{ AR now 410 MSB per dB.Translated to Hz, by 32768 MSB per 4800 Hz
	  (.14648 Hz/MSB), the scale is 60 Hz per dB.
	  The scaling constant, k_v2f is 1*128 to 255*128 (.00390625 to
	  0.99609) due to *128 scaling after receiving the 'K' command.
	 }
        my0=dm(k_v2f);			{ Scaling constant }
        mr=ar*my0 (SS);			{ Scale it }
	{ Shift left 6 bits (*64) to bring scaling up to 64*60=3840 max
	  Hz per dB. Minimum is 3840/255=15.05 Hz/dB.
	}
	sr=ashift mr1 by 6 (HI);
	sr=sr or lshift mr0 by 6 (LO);	{ SR1 now 64 times bigger }
	{ Put overflow stuff here }
 
	{ Add in a base frequency, such as 450 Hz }
	ay0=dm(basef_v2f);
	ar=sr1+ay0;
	if ge jump v2f1;
	ar=0;				{ No neg freqs }
	{ ar now has the new frequency, scaled 0 to 32767 for 0 to 4800 Hz.
	  This is the phase increment for the sin routine. }
v2f1:	dm(v2f_save_freq)=ar;		{ Use between updates, for sig source }
v2f11:	ay0=dm(phase_v2f);		{ Last phase value }
	ar=ar+ay0;			{ Increment ph, allowing wrap-around }
	dm(phase_v2f)=ar;		{ Save for next time }
	ax0=ar;
	{ The input to sin is phase angle, scaled (-32768, 32767) for
	  (-180.0,179.994) degrees. }
	call sin;			{ Phase in ax0, Sin returned in AR }
	{ Output the v2f tone, after setting level to pleasant value }
	my0 = dm(v2f_af_level);
	mr=ar*my0(SS);			{ Use sidetone level, result in mr1 }

bl4fff:	i0=dm(fir2_i0_sav); m0=1; l0=TAPS2;  { Put audio into output line }
	dm(i0,m0)=mr1;
	dm(fir2_i0_sav)=i0;
	jump end_blocks;

{ b4v2f2 picks up the peak signal power, found by searching over bins,
  in f2n1024b.dsp. This is checked more often than new data is available, and
  only used once by checking v2f_bin_ready.  }
b4v2f2:	ax0=dm(v2f_bin_ready);
	ar=tstbit 0 of ax0;
	if eq jump b4v2f7;
	mr0=dm(v2f_pkl);		{ Data goes in mr0 and mr1 for log }
	mr1=dm(v2f_pkh);
	ax0=0;  dm(v2f_bin_ready)=ax0;	{ Don't stop here again, til new data }
	jump b4v2f1;

{ b4v2f3 picks up the average signal power, found by searching over bins,
  in f2n1024b.dsp.  Similar to b4v2f2. }
b4v2f3:	ax0=dm(v2f_bin_ready);
	ar=tstbit 0 of ax0;
	if eq jump b4v2f7;
	mr0=dm(v2f_pwrl);
	mr1=dm(v2f_pwrh);
	ax0=0;  dm(v2f_bin_ready)=ax0;
	jump b4v2f1;

{ Here because we need to update the V2F sine wave, but no pwr update is
  complete (only every 512 sine points). Use the last frequency, saved
  in v2f_save_freq.    }
b4v2f7: ar=dm(v2f_save_freq);
	jump v2f11;

{ pwr_v2f is used to take the 48 kHz data samples v_v2f1, v_v2f2,
  ..., v_v2f5, HP flter them to remove A/D DC offset, then square and add to
  to the power sum. The new data sample comes in MX1. Power is accumulated
  in 32-bit pwr_v2f_l and _h.
  HP_B0=27054 (.82562) and HP_A1=-18762 (-.57257) running at 48 kHz
  is down 26 dB at 200 Hz and flat within +/- 0.25 dB across 10 to 20 kHz.  }
pwr_v2f:
	my0=HP_B0;			{ High Pass with a single pole }
	mr=mx1*my0 (SS);		{ New data * B0 }
	mx0=dm(last_hp_x);		{ Previous data sample }
	mr=mr-mx0*my0 (SS);		{ - Last data * B0 }
	dm(last_hp_x)=mx1;		{ Save for next time }
	mx0=dm(last_hp_y);		{ Previous output of HPF }
	my0=HP_A1;
	mr=mr+mx0*my0 (SS);		{ + Last output * A1, now in mr1 } 
	dm(last_hp_y)=mr1;		{ Result for next run of HPF }

	my0=mr1;			{ Square to get power }
	mr=mr1*my0 (SS);

	{ This is now a 32-bit number representing power. Add it to the 
	  current running power sum. }
	ax0=dm(pwr_v2f_l); ay0=mr0; ar=ax0+ay0;	{ Accumulate power (32 bit) }
	dm(pwr_v2f_l)=ar;
	ax0=dm(pwr_v2f_h); ay0=mr1; ar=ax0+ay0+c;
	dm(pwr_v2f_h)=ar;
	rts;				{ Power in pwr_v2f_l and _h }

{ mult32 does a 32 bit signed product of (mx1,mx0) and (my1,my0)
  (High Bits, Low Bits) leaving the result in (mr1,mr0). ADI Vol 1, List 2.7 }
mult32:	mr=mx0*my0 (UU);		{ Low order words }
	mr0=mr1;			{ Shift right 16 bits }
	mr1=mr2;
	mr=mr+mx1*my0 (SU);		{ Two cross terms }
	mr=mr+mx0*my1 (US);
	mr0=mr1;			{ Shift right 16 bits }
	mr1=mr2;
	mr=mr+mx1*my1 (SS);		{ High 16 bit term }
	rts;

#if 0
{The following are #if 0'd as they are not being used. Routines are OK }
{ div_s  -  Signed single-precision division.
  af=msw of numerator, ay0=lsw of numerator, ax0=denom
  Returns 16-bit result in ay0. See ADI Vol 1, p18.
  Call, for example, by: ax0=12345; af=pass ax0; ay0=0; ax0=mr1; call div_s;
}
div_s:	divs af,ax0;			{ Get sign bit }
	jump div1;

{ div_uns  -  Unsigned single-precision division.
  af=msw of numerator, ay0=lsw of numerator, ax0=denom
  Returns 16-bit result in ay0. See ADI Vol 1, p18.  }
div_uns:
	ASTAT=0;			{ Clear AQ of ASTAT }
        divq ax0;
div1:   divq ax0; divq ax0; divq ax0;
        divq ax0; divq ax0; divq ax0;
        divq ax0; divq ax0; divq ax0;
        divq ax0; divq ax0; divq ax0;
        divq ax0; divq ax0; divq ax0;
        rts;				{ Qoutient in ay0 }

{ byte2i - Mini floating point. Transmit wider dynamic range in a single byte.
    bit  7   Sign bit (0 is pos)
    bits 6-4 Mantissa, 8/16 to 15/16 in 1/16 steps
    bits 3-0 Exponent, power of 2
  This routine takes the byte as the low order 8 bits of AX0 and converts
  to a 16 bit 1.15 signed fixed point number.
  Return value is in MR1.
  For example, 01010011 would have sign bit=0 (positive), mantissa=5
  (101 binary) and exponent=3 (0011 binary). This converts to +(5+8)/16 times
  2^3 or +8.
  The biggest positive number is 15/16 times (2^15-1) or 30719 (AX0=7F).
}
byte2i:	ay0=0X000F;
	ar=ax0 and ay0;			{ This is the exponent }
	se=ar;				{ Amount of left (up) shift }
	ar=0XFFFF;			{ All 1's to shift into SR1 }
	sr=lshift ar (lo);
	my0=sr1;			{ Save for later, free up sr }
	ay0=0X0070;			{ Extract the mantissa }
	ar=ax0 and ay0;
	sr=lshift ar by 7 (HI); 	{ Move into bits 13,12,11 }
	ay0=0X4000;			{ Add "1/2" }
	ar=sr1+ay0;
	{ At this point, AR ranges from 16384 to 32720 in steps of 2048,
	  which in 1.15 fixed point is 0.5 to 0.9375 in steps of 0.0625 }
	mr=ar*my0 (SS);			{ Combine exponent and mantissa }
	ar=tstbit 7 of ax0;		{ Sign bit }
	if eq jump b2i_1;		{ 0 is positive }
	ar=-mr1;			{ Change sign }
	mr1=ar;
b2i_1:	rts;
#endif

block5: ar=dm(save_if);			{ A/D sample }
	dm(v_v2f5)=ar;			{ Save data for v2f }
	add_point;
bin_bl5:				{ Block for host communications }
	ax0 = 0;			{ Now reset block count for 48/5 rate }
	dm(block_count) = ax0;
xbl5:					{ Entry point from xmit routines }
					{ Starting here is common T & R }

	{ Possible reentry to monitor here. Checks
	  to see if there are exactly three bytes in command buffer. If so,
	  see if bytes 2 and 3 are either '$$' or 'OK'. If so set these into
	  ar register, clean up DSP-10 process and jump to mon_start.
	  This is all for EPROMable version.  Added 2.1 6 Dec 2001 RSL }
#if EPROM
	ax0=dm(uart_rx_cmd_count);	{ Must have 3 command bytes }
	ay0=3;
	none=ax0-ay0;			{ Check if 3 bytes }
	if ne jump bl5ac;		{ Not 3 }
	{ Byte 0='$' always. Test if Bytes 1 and 2 make up '$$' or 'OK' }
	ar=dm(cmd_buf);			{ Byte 1 (first '$' not in cmd_buf) }
	sr=lshift ar by 8 (hi);		{ Move second byte into high 8 bits }
	ay0=dm(cmd_buf+1);
	ar=sr1 or ay0;			{ 16 bit version of command }
	ay0=0x2424;			{ Test for '$$' }
	none=ar-ay0;
	if eq jump go_to_mon;
	ay0=0x4f4b;			{ Test for 'OK' }
	none=ar-ay0;
	if eq jump go_to_mon;
bl5ac:
#endif
	call do_command;		{ See if 6 bytes rcvd, and process }

	{ serial_timeout is between 10 and 20 when a command is being received.
	  It is  -1 if no command is being received. A value of zero means that
	  a command was interrupted and timed out, after a wait of 1 msec.
	}
	ar=dm(serial_timeout);		{ See what the value is }
	none=pass ar;
	if lt jump bl5aa;		{ -1 is OK }
	ar=ar-1;			{ Continue the timeout count down }
	dm(serial_timeout)=ar;		{ Save for next time }
	if gt jump bl5aa;		{ Values above zero are OK }
	{ A command error has occurred, timeout got to zero; process it }
	ax0=-1;
	dm(serial_timeout)=ax0;		{ Stop further errors }
	ax0=0;
	dm(UART_rx_cmd_count)=ax0;	{ Cancel any command fragments }
	dm(UART_rx_cur_bit)=ax0;	{ Start new byte }
	ax1=^cmd_buf;			{ Reset buffer pointer }
	dm(UART_rx_cmd_buf_ptr)=ax1;	{ Ready to receive command }

	ar=dm(serial_error);		{ To be sent to PC at next send }
	ar=ar+1;			{ Bump error count }
	ay0=0X001F;			{ Clear error code }
	ar=ar and ay0;
	ay0=0X0020;			{ Code for timeout error }
	ar=ar or ay0;
	dm(serial_error)=ar;		{ Put back to send to PC }
bl5aa:	jump end_blocks;

{ xmt_isr  -  Routine jumped to when in transmit. Comes with audio signal 
  in mx0.  Handles CW, USB, LSB now.
  Relevant control words are:
     mode_select  0=SSB, 1=CW, 2=FM
     ssb_select   0=LSB,  1=USB
     t_r_control  Bit 0=1 for dsp xmit.
     t_r_control  Bit 4=1 for cw key down				}
xmt_isr:
	ax1=dm(mode_select);
	ay1=1;
	ar=ax1-ay1;
	if lt jump xi_ssb;
	if eq jump xi_cw;
	ar=ar-ay1;
	if eq jump xi_fm;
	jump end_blocks;		{ Leave structure for other modes }

	{ Entry point for finishing up transmit interrupt service }
xmt_isr1:
	call uart_rcvr;			{ See if character is coming in }
	ay0 = 1;
	ar = dm(block_count);		{ Ranges from 0 to 4 }
	ar = ar + ay0;			{ Update to next block }
	dm(block_count) = ar;		{ Save for next interrupt, now 1 to 5 }
	ar = ar - ay0;			{ Test for current block }
	if eq jump x_block1;
	ar = ar - ay0;
	if eq jump x_block2;
	ar = ar - ay0;
	if eq jump x_block3;
	ar = ar - ay0;
	if eq jump x_block4;
	ar = ar - ay0;
	{ x_block5 is entry point for shared T/R code that, among other things
	  receives commands from the PC }
	jump x_block5;			{ This block will reset count to 0 }

xi_cw:
	{ If key is down, put a .9 (29491) into CW fir filt.
	  Modulate fir output onto carrier.  This scheme
	  allows top space for overshoot in the fir.			}
	ax0 = dm(PFDATA);		{ Get hardware CW key data }
	af=tstbit 5 of ax0;
	if eq jump xi_cw0;		{ CW key is down }
	ar = 0;				{ Now see if 'software' key is down }
	ax0 = dm(t_r_control);
	af = tstbit 4 of ax0;
	if eq jump xi_cw1;		{ Jump if key is up }
xi_cw0:	{ Output sidetone to speakers }
	ax1 = dm(sdtone_dph);		{ Phase increment for sidetone }
	ay1 = dm(sdtone_ph);		{ Last phase }
	ar = ax1 + ay1;			{ New phase }
	ax0 = ar;
	dm(sdtone_ph) = ar;		{ For next time }
	call sin;			{ Phase in ax0, Sin returned in AR }
	my0 = dm(sdtone_ampl);
	mr=ar*my0(SS);			{ Gate sidetone }


#if CODEC_1847
	dm(tx_buf+2) = mr1;		{ Right speaker only }
#endif

#if CODEC_AIC23
	dm(tx_buf+0) = mr1;
#endif

	ar = 29491;			{ 0.9 to key click filter }
xi_cw1:	call fir_xmt_cw;		{ Input in ar, output in mr1 }
	my0 = dm(xmit_power);		{ Xmit_gain (power control) }
	mr = mr1 * my0 (SS);		{ Power controlled modulating signal }
	my0 = mr1;
	ax0 = dm(if_dphase);		{ Phase increment for lo }
	ay0 = dm(if_phase);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	dm(if_phase) = ar;		{ For next time }
	ax0 = ar;  call sin;		{ Phase in ax0, Sin returned in AR }
	mr=ar*my0(SS);			{ CW Gate }

	{ Add in Gaussian white noise if random_lev != 0. }
	{ Added rev 2.1 }
	ax0 = dm(random_lev);
	none = pass ax0;
	if eq jump xi_cw1a;		{ Not using noise }
	dm(temp1) = mr1;		{ Save from getrnd() }
	call getrnd;			{ Return GWN in af }
	ar = pass af;
	my0 = dm(random_lev);
	mr = ar * my0 (SS);		{ Level adjusted RN in mr1 }
	ay0 = dm(temp1);
	ar = mr1 + ay0;			{ Sine wave plus noise }
	mr1 = ar;			{ Back where we were before noise }

xi_cw1a:
	ar = abs mr1;			{ Hunt for largest magnitude output }
	ay0 = dm(dac_big);		{  to be sent to pc }
	none = ar - ay0;
	if lt jump xi_cw2;		{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }
xi_cw2:
	{ Note: tx_buf+1 happens to be both 1847 and AIC23 }
	dm(tx_buf + 1) = mr1;		{ transmit i-f sig out }
	jump xmt_isr1;

{ xi_ssb  -  All of the routines associated with ssb transmit.
  These are 48 kHz rate.
  Transmit audio comes from the the left adc (mx0) and is run through 
  hilbert3 to create I and Q components.  Two sine waves (0 and 90 degrees)
  are created in 14 to 16 kHz region and mixed with the I and Q audio.
  Output to the dac is in the left channel.				}
xi_ssb:
	{  Note we are clippimg, filtering and then generating ssb. This is
	  not the best and one should avoid too much of the clipping region
	  when using ssb.  Audio comes in mx0. }

	{ The mike gain is divided into a multiplier (attenuator) and a shift
	  from mgain_shift. All from the 'M' command. In addition, there is a
	  fixed gain of 2 in the FIR filter that follows. This provides some
	  additional gain for low output mikes. }

	my0 = dm(mike_gain);
	mr = mx0*my0 (SS);		{ adc output reduced by mike gain }
	se = dm(mgain_shift);
	sr = ashift mr1 (hi);
	ar=sr1;				{ The audio signal }

	{ The fir_xmt_audio filter includes the equalizer. Put it here so
	  that the clipper sees "flat" audio. Particlarly a problem if
	  low freqs are reduced in the equalizer, as these are the ones with
	  x3 and x5 distortion when clipping.}
xiss0:	call fir_xmt_audio; 		{ Input in ar, out in mr1, Av=2 }

	{ The clipping is done at a level of 8192.
	  This will be multiplied by 4 following the limiter.
	  Audio now in mr1.       }
	ay1 = mr1;			{ Remember for later }
	ay0 = 8192;			{ The clipping level }
	none = mr1 - ay0;		{ Compare levels }
	if lt jump xis_tstlow;
	mr1 = 8192;			{ Do high clipping }
	jump xis_endclip;
xis_tstlow:
	none = mr1 + ay0;		{ Compare negative side }
	if gt jump xis_endclip;
	mr1 = -8192;
xis_endclip:				{ Clipped waveform now in mr1 }

	{ Monitor amount of clipping to send to PC }
	ar=mr1-ay1;			{ This is 0 w/o clipping. }

	ar = abs ar;			{ Hunt for largest magnitude output }
	ay0 = dm(del_clip_big);		{  to be sent to pc }
	none = ar - ay0;
	if lt jump del_clip1;		{ Not as big as previous biggest }
	dm(del_clip_big) = ar;		{ Save for next time }
del_clip1:

	{ Power setting follows the limiter. Otherwise, it is the
	  same as the mike gain!  }
	my0=dm(xmit_power);		{ 0, 32767  }
	mr=mr1*my0 (SS);

	{ wfir1i (borrow): 2.8 kHz w/48 kHz sample, In mr1, Out mr1 Av=2 }
	call wfir1i;

	{ do_hilbert3 does 90 degree and delay. Input latest data point
	  in mr1 and output I (delayed) in ax1 and Q (FIR) in mr1    }
	call do_hilbert3;

	si = mr1;			{ Save in si for Q mixer }
	ax0 = dm(if_dphase);		{ Phase increment for lo }
	ay0 = dm(if_phase);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	dm(if_phase) = ar;		{ For next time }
	ax0 = ar;  call sin;		{ Phase in ax0, Sin returned in ar }
	my0 = ax1;			{ I signal is in ax1 }
	mr=ar*my0(SS);			{ I mixer }
	ay1 = mr1;			{ Save mixer output }
	ax0 = dm(if_phase);
	ay0 = 16384;			{ 90 degrees for quadrature lo }
	ar = ax0 + ay0;
	ax0 = ar;  call sin;		{ Cos lo signal, sin() preserves my0 }
	my0 = si;			{ Q signal is in si }
	mr = ar*my0(SS);		{ Q mixer }
	ar = mr1 + ay1;			{ lsb signal }
	ax0=dm(ssb_select);		{ 0=lsb, 1=usb }
	none=pass ax0;
	if eq jump xis0;
	ar = mr1 - ay1;			{ usb }
xis0:	ax1=ar;				{ For outputting }

	{ For ssb transmit, monitor at x0.5 point to allow an overload
	  indicator in the PC }
	ar = abs ar;			{ Hunt for largest magnitude output }
	ay0 = dm(dac_big);		{  to be sent to pc }
	none = ar - ay0;
	if lt jump xis1;		{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }
xis1:
	{ The SSB signal to this point has been kept at 15 bits to prevent
	  clipping in the filters or Hilbert, which could be very bad.
	  Now, raise the signal up one bit to allow the full DAC range to
	  be available.	 Use xmit_power (in PC) to trim back for o'shoot. }
	ena AR_SAT;			{ Last-gasp clipper }
	ay1 = ax1;
	ar = ax1 + ay1;			{ Last needed x2 }
	dis AR_SAT;
	{ Output in ar in 8 to 22 kHz range at 48 KHz sample rate.      }
	ax1=dm(out_steer);
	none=pass ax1;
	if gt jump out_right;
	dm(tx_buf + 1) = ar;		{ I-F sig out }
	jump xmt_isr1;

out_right:	
#if CODEC_1847
	dm(tx_buf+2) = ar;		{ Alternate I-F sig out }
#endif
#if CODEC_AIC23
	dm(tx_buf+0) = ar;
#endif
	jump xmt_isr1;

{ xi_fm  -  All of the routines associated with fm transmit.
  These are 48 kHz rate.
  Transmit audio comes from the the left adc (mx0).
  Audio gain of 1 to here. To allow some gain in this path, put 
  mike gain here (0, 32767/32768) and follow with x8.  Dynamic
  range is not a problem here.
  Note we are clippimg, filtering and then generating fm.
  Output to the dac is in the left channel.     }
xi_fm:	{ The mike gain is divided into a multiplier (attenuator) and a shift
	  from mgain_shift. All from the 'M' command. In addition, there is a
	  fixed gain of 2 in the FIR filter that follows. This provides some
	  additional gain for low output mikes. }

	my0 = dm(mike_gain);
	mr = mx0*my0 (SS);		{ adc output reduced by mike gain }
	se = dm(mgain_shift);
	sr = ashift mr1 (hi);		{ Audio signal in sr1 }

	{ The fir_xmt_audio filter includes the equalizer. Put it here so
	  that the clipper sees "flat" audio. Particlarly a problem if
	  low freqs are reduced in the equalizer, as these are the ones with
	  x3 and x5 distortion when clipping.}
	ar=sr1;  {1 instr could be saved. also an extra ar=sr1 in fir routine }
	call fir_xmt_audio; 		{ Input ar, out mr1, Av=2, saves ax1 }

	{ As near as I can tell, (see recent ARRL handbooks) the pre-emphasis
	  and de-emphasis for NBFM is throughout the audio band. The handbooks
	  show 1000 uSec (159 Hz 3 dB single pole) and the Japanese radios
	  that I measured in reception support this concept. Going with
	  this, we install a HP filter, single pole (RC), with a 3 dB point
	  at 2880 Hz. y=b1*x+b2*xlast-a2*ylast  }
	mx1 = mr1;
	my0 = 27518;			{ b1=0.8398 }
	mr = mx1*my0 (ss);		{ x*b1 }
	mx0 = dm(xlast);
	mr = mr - mx0*my0 (ss);		{ subtract b2*xlast }
	dm(xlast) = mx1;		{ For next time }
	my0 = 22268;			{ a2=0.6796 }
	mx0 = dm(ylast);
	mr = mr - mx0*my0 (ss);		{ subtract a2*ylast, the feedback }
	dm(ylast) = mr1;		{ Save for next time }

	{ This cuts a voice by at quite a few dB: }
	sr = ashift mr1 by 4 (hi);	{ Audio in sr1 }

	{ Two tones are available in FM. These can be with or without
	  voice, as per dm(ttn_on). Set by 'X' command.
	  The tones are put after the preemphasis and the mike
	  equalizer so that low subaudible tones can be used. The levels
	  of all tones should be cut at low frequencies according
	  to the preemphasis of 6-dB/octave.   }
	{NOTE: Bit 2 of dm(ttn_on) must be 1 for non-tone operation }
	ax1=dm(ttn_on);
	ar=tstbit 2 of ax1;		{ Bit 2=1 means leave/add-in voice }
	if gt jump xf_tn0;		{ Add in voice if bit is 1 }
	sr1=0;
xf_tn0:	dm(tmp1)=sr1;			{ Build signal in tmp1 }

	{ Transmit audio tone(s). }
	ar=tstbit 0 of ax1;		{ dm(ttn_on) still there }
	if eq jump xf_tn1;		{ 0=Tone 1 not being used }
	{ Generate tone 1 }
	ax0 = dm(ttn_freq1);		{ Phase increment for tone 1 }
	ay0 = dm(ttn_ph1);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	dm(ttn_ph1) = ar;		{ For next time }
	ax0=ar; call sin;		{ Phase in ax0, Sin returned in AR }
	my0=dm(ttn_level1);		{ Sets tone level }
	mr=ar*my0 (ss);			{ Scaled tone 1 ends in mr1 }
	ay0=dm(tmp1);			{ Audio signal being built }
	ar=mr1+ay0;
	dm(tmp1)=ar;			{ Tone + possibly voice }
xf_tn1:					{ Audio is in dm(tmp1) }
	ar=tstbit 1 of ax1;		{ See if tone 2 is needed }
	if eq jump xf_tn3;		{ No... }
	{ Generate tone 2 }
	ax0 = dm(ttn_freq2);		{ Phase increment for tone 1 }
	ay0 = dm(ttn_ph2);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	dm(ttn_ph2) = ar;		{ For next time }
	ax0=ar; call sin;		{ Phase in ax0, Sin returned in AR }
	my0=dm(ttn_level2);
	mr=ar*my0 (ss);
	ay0=dm(tmp1);			{ Audio signal being built }
	ar=mr1+ay0;			{ Up to two tones and voice }
	dm(tmp1)=ar;

xf_tn3:	mr1=dm(tmp1);			{ Get tone(s)+voice }

	{ The clipping is done at a level of 8192.
	  This will be multiplied by 4 following the limiter.
	  Audio now in mr1.       }
	ay1 = mr1;			{ Remember for later }
	ay0 = 8192;			{ The clipping level }
	none = mr1 - ay0;		{ Compare levels }
	if lt jump xif_tstlow;
	mr1 = 8192;			{ Do high clipping }
	jump xif_endclip;
xif_tstlow:
	none = mr1 + ay0;		{ Compare negative side }
	if gt jump xif_endclip;
	mr1 = -8192;
xif_endclip:				{ Clipped waveform now in mr1 }

	{ Monitor amount of clipping to send to PC }
	ar=mr1-ay1;			{ This is 0 w/o clipping. }
	ar = abs ar;			{ Hunt for largest magnitude output }
	ay0 = dm(del_clip_big);		{  to be sent to pc }
	none = ar - ay0;
	if lt jump fdel_clip1;		{ Not as big as previous biggest }
	dm(del_clip_big) = ar;		{ Save for next time }
fdel_clip1:				{ Sig still in mr1 }

	{ wfir1i (borrow): 2.8 kHz w/48 kHz sample, In mr1, Out mr1 Av=2 }
	call wfir1i;

	{ The signal level here, after clipping and filtering is
	  +/-16384 and a small amount for filter overshoot. In theory, to scale
	  16384 to 5 kHz deviation, multiply by 13650. Including the overshoot
	  makes 12996=114^2 closer to 5 kHz peak }
	my0=dm(fm_dev);			{ Scale full range to 10 kHz dev }
	mr=mr1*my0(SS);
	ax0 = dm(if_dphase);		{ Phase increment for lo }
	ay0 = dm(if_phase);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	ay0 = ar;
	ar = mr1 + ay0;			{ Add in fm }
	dm(if_phase) = ar;		{ For next time }
	ax0 = ar;  call sin;		{ Phase in ax0, Sin returned in ar }
	my0 = dm(xmit_power);		{ Xmit_gain (power control) }
	mr = ar * my0 (SS);		{ Power controlled modulating signal }

	ar = abs mr1;			{ Hunt for largest magnitude output }
	ay0 = dm(dac_big);		{  to be sent to pc }
	none = ar - ay0;
	if lt jump xifm1;		{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }
xifm1:
	{ Note: tx_buf1 happens to be the same for both 1847 and AIC23 }
	dm(tx_buf + 1) = mr1;		{ transmit i-f sig out }
	jump xmt_isr1;

{***************************************************************************
 * Transmit Block subroutines - Called by the 48KHz interrupt routine
 * one after another at a 9.6 KHz rate. Max of xxx cycles per block.
 ***************************************************************************}
x_block1:
	jump end_blocks;
x_block2:
	jump xbl2;			{ Transmit serial data to PC }
x_block3:
	jump end_blocks;
x_block4:
	jump end_blocks;
x_block5:
	ax0 = 0;			{ Now reset block count for 48/5 rate }
	dm(block_count) = ax0;
	jump xbl5;			{ Send serial port data & check cmds }

{********************   End  ISR's    *************************************}

{***************************************************************************
 * Do_command    Checks if new command from host is available and if so
 * processes it.  Command ready flagged by UART_rx_cmd_count=6;
 *          RSL - 24 June 96
 ***************************************************************************}
do_command:
	ax0=dm(UART_rx_cmd_count);
	ay0=6;
	none=ax0-ay0;
	if lt rts;			{ No cmd is ready }

	ax0=-1;				{ Prevent timeout error }
	dm(serial_timeout)=ax0;		{ This is the idle state for timeout }

	ax1=dm(cmd_buf);		{ This is used from ax1 later }
	dm(status_buf+2)=ax1;		{ Return command for diagnostics }

        ay0 = 1;			{ Download dm;  PC-->DSP }
        none = ax1 - ay0;
        if eq jump dnld_dm;

        ay0 = 2;			{ Download pm, msb 16 bits }
        none = ax1 - ay0;
        if eq jump dnld_pm_msb;

        ay0 = 3;			{ Download pm, lsb 8 bits }
        none = ax1 - ay0;
        if eq jump dnld_pm_lsb;

        ay0 = 4;			{ Upload dm;  DSP-->PC }
        none = ax1 - ay0;
        if eq jump upld_dm;

        ay0 = 5;			{ Upload pm, 24 bits }
        none = ax1 - ay0;
        if eq jump upld_pm;

        ay0 = 7;			{ Determine SR enable line, by byte 2 }
        none = ax1 - ay0;
        if eq jump enable_line7;

        ay0 = 8;			{ Determine Shift Reg 16 bits }
        none = ax1 - ay0;
        if eq jump shift_16bits;

        ay0 = 9;			{ Data format to send host (was 'D') }
        none = ax1 - ay0;
        if eq jump set_tr_format;

        ay0 = 10;			{ AF Gain (was 71 'G') }
        none = ax1 - ay0;
        if eq jump set_af_gain;

        ay0 = 65;   { 'A' }		{ Audio Filter: 0=off, 1=On, }
        none = ax1 - ay0;		{          LMS: 0=off, 1=on }
        if eq jump audio_fil_sw;

        ay0 = 66;   { 'B' }		{ Audio Filter per  }
        none = ax1 - ay0;
        if eq jump audio_fil_choose;

        ay0 = 67;   { 'C' }		{ LMS Filter, Beta }
        none = ax1 - ay0;
        if eq jump LMS_set;

	{ 'D' is reserved for Monitor commands }

        ay0 = 69;   { 'E' }		{ Delay of right channel }
        none = ax1 - ay0;
        if eq jump set_delay_right;

        ay0 = 70;   { 'F' }		{ FFT control }
        none = ax1 - ay0;
        if eq jump set_fft_fcn;

	{ 'G' is reserved for monitor commands }

        ay0 = 72;   { 'H' }		{ CW Sidetone }
        none = ax1 - ay0;
        if eq jump set_sidetone;

        ay0 = 73;   { 'I' }		{ RF and IF Gains, 16 bit }
        none = ax1 - ay0;
        if eq jump set_r_if_gain;

        ay0 = 74;   { 'J' }		{ Sets out_steer }
        none = ax1 - ay0;
        if eq jump set_in_sel;

        ay0 = 75;   { 'K' }		{ Controls pwr-to-tone, V2F }
        none = ax1 - ay0;
        if eq jump set_p2t;

        ay0 = 77;   { 'M' }		{ Mode: lsb, usb, cw & mike_gain }
        none = ax1 - ay0;
        if eq jump set_mode;

        ay0 = 78;   { 'N' }		{ Noise added onto CW for test sigs }
        none = ax1 - ay0;
        if eq jump set_noise;

        ay0 = 82;   { 'R' }		{ Reset functions }
        none = ax1 - ay0;
        if eq jump do_reset;

        ay0 = 83;   { 'S' }		{ Squelch for rcvr, deviation for FM }
        none = ax1 - ay0;
        if eq jump set_squelch;

        ay0 = 84;   { 'T' }		{ T/R Ctrl, CW Key & xmit_power }
        none = ax1 - ay0;
        if eq jump set_t_r;

        ay0 = 86;   { 'V' }		{ AGC (like AVC) 0=off }
        none = ax1 - ay0;
        if eq jump set_agc;

        ay0 = 87;   { 'W' }		{ Window 0=off, 1=Hamming, ... }
        none = ax1 - ay0;
        if eq jump window_cmd;

        ay0 = 88;   { 'X' }		{ Transmit audio tone }
        none = ax1 - ay0;
        if eq jump ttone_cmd;

cmd_end:
	ax0=0;				{ Flags to start looking for new cmd}
	dm(UART_rx_cmd_count)=ax0;
	rts;

{ Cmd 1 -    ESC 1 adrl adrh datal datah }
{ Puts data into dm at adr }
dnld_dm:				{ Command 1 }
	call get_cmd_data;		{ Returns with adr in i4 }
	m4=0; l4=0;
	dm(i4, m4)=mr0;			{ mr0 has data }
	jump cmd_end;

dnld_pm_msb:				{ Command 2, Download 16 msb to pm }
	call get_cmd_data;		{ Get pm adr and 16 msb to mr0 }
	m4=0;  l4=0;
	ar=pm(i4, m4);			{ This puts 8 lsb into px }
	pm(i4, m4)=mr0;			{ Write 16 msb & rewrite 8lsb from px }
	jump cmd_end;

dnld_pm_lsb:				{ Command 3, Download 8 lsb to pm }
	mr0=dm(cmd_buf+1);		{ Get the pm address }
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	i4=mr0;				{ Put it in i4 }
	m4=0;  l4=0;
	ar=pm(i4, m4);			{ Put the 16 msb in ar }
	px=dm(cmd_buf+3);		{ Get the 8  lsb }
	pm(i4, m4)=ar;			{ Rewrite msb16 and add new 8 from px }
	jump cmd_end;

{ Uploading data from the DSP to the PC is done with the 4 and 5 commands.
  Bytes 1 & 2 have the address and 3 has a tag to let the PC identify
  which data has arrived. This tag should be changed by the PC program
  for each new word that is requested. }
upld_dm:				{ Command 4 }
	call get_cmd_data;		{ Adr in i4 }
	dm(status_buf+13)=mr0;		{ Tag from cmd bytes 3 goes in +13}
	m4=0; l4=0;
	ar=dm(i4, m4);
	dm(status_buf+7)=ar;		{ Low byte in first 8 bits }
	sr=lshift ar by -8 (hi);	{ Get high byte }
	dm(status_buf+8)=sr1;
	ax0=dm(fft_fcn);		{ Now software trigger data send }
	ar=setbit 10 of ax0;		{ This is reset by sending of data }
	dm(fft_fcn)=ar;			{ Trigger it }
	jump cmd_end;

	{ Bytes 1,2 have pm address, 3 is tag, return 24-bit pm contents+tag }
upld_pm:				{ Command 5, Upload from pm }
	call get_cmd_data;		{ Get pm adr and 16 msb to mr0 }
	dm(status_buf+13)=mr0;		{ Tag from cmd byte 3 goes in +13}
	m4=0; l4=0;
	ar=pm(i4, m4);			{ Addr came from cmd bytes 1,2 }
	dm(status_buf+7)=ar;		{ Middle byte (in first 8 bits of +7) }
	sr=lshift ar by -8 (hi);	{ Get high byte }
	dm(status_buf+8)=sr1;		{ High byte }
	dm(status_buf+18)=px;		{ Low byte }
	ax0=dm(fft_fcn);		{ Now software trigger data send }
	ar=setbit 10 of ax0;		{ This is reset by sending of data }
	dm(fft_fcn)=ar;			{ Trigger it }
	jump cmd_end;

	{ The enable line is set to the value of the lsb in cmd_buf+1.
	  PF7 is the enable line for the shift regs & 126MHz synth.  Also sets
	  phase increment for the dsp 2nd LO so that frequencies shift 
	  in sync.  }
enable_line7:
	reset fl2;			{ Clock must be low for LMX1501 }
	mr0=dm(cmd_buf+3);		{ Get low bytes of data for the dPhi }
	mx0=dm(cmd_buf+4);		{   and the high byte }
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(if_dphase)=mr0;		{ 16 bits of delta-phase data }
	ax0=dm(cmd_buf+1);
	ar=tstbit 0 of ax0;		{ See if set or clear of PF7 }
	ax0=dm(PFDATA);			{ Get ready to work on it }
	if eq jump el7;			{ Clears bit 7 of PFDATA }
	ar=setbit 7 of ax0;		{ Here to set bit 7 }
	dm(PFDATA)=ar;
	jump cmd_end;
el7:	ar=clrbit 7 of ax0;
	dm(PFDATA)=ar;
	jump cmd_end;

	{ Get 16 bits of data for the shift registers and U104.
	  This can be shifted out in 16*3*104=4992 uSec which is less
	  than the time required to receive the next command
	  (6*1042=6252 uSec.) }
shift_16bits:
	reset fl2;			{ Make sure clock is low  to start }
	mr0=dm(cmd_buf+1);		{ Get low bytes of data for the SR }
	mx0=dm(cmd_buf+2);		{   and the high byte }
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(sr_data)=mr0;		{ 16 bits of shift data }
	ax0=63;				{ Set up flag/count combo, 4*16-1 }
	dm(sr_count)=ax0;
	jump cmd_end;

{ Byte 2 is LMS control that is sent from PC as (0,1,2) to be a positive
  1 byte number. It is made  into a 16 bit DSP integer and then lowered
  by 1 for control convenience in the interrupt routine. The lowered
  values for dm(audio_LMS_on) are: 0=Off, 1=Denoise, -1=Autonotch }
audio_fil_sw:				{ 'A' Audio fir off/on, LMS off/on }
	ax0=dm(cmd_buf+1);
	dm(audio_fir_on)=ax0;
	ay0=dm(cmd_buf+2);
	ar=ay0-1;			{ Decrement as explained above }
	dm(audio_LMS_on)=ar;
	jump cmd_end;

{ 'B'  -  audio_fil_choose selects the FIR filter coefficients.
  Byte 1 is Audio filter number, 0 to 7.
  Byte 2 is gain 1 to 128, specified by 0 to 7 where A=2^byte2  }
audio_fil_choose:
	ax0=dm(cmd_buf+1);
	dm(audio_fir_select)=ax0;
	ax0=dm(cmd_buf+2);
	ay0=0X0007;
	ar=ax0 and ay0;
	ar=-ar;
	dm(audio_fir_gain)=ar;
	jump cmd_end;

{ 9 Sets the format that spectral data is sent back to the host pc.
  Byte 1: Bit 0  0=8 bit data, 512 bytes
                 1=16 bit data, 256 words
          Bit 1  0=Send FFT data, 'A' stream
                 1=Suppress FFT data, send only status data, 'B' stream
  Byte 2 is a shift value, power of 2
  Bytes 3 and 4 are an offset applied to 8 bit log data.  }
set_tr_format:
	ax0=0X0041;			{ 'A' }
	dm(esc_a+1)=ax0;		{ Standard 'A' data stream }
	ax0=dm(cmd_buf+1);		{ Bit 0: 0=8bit x 512,  1=16bit x 256 }
					{ Bit 1: 0=std data stream, 0='B' type}
	dm(data_format)=ax0;
	af=tstbit 1 of ax0;
	if eq jump strf0;
	ax0=0X0042;			{ 'B' }
	dm(esc_a+1)=ax0;		{ Data stream will be: ESC 'B' status }
	ax0=0;				{ Here to suppress FFT transmission }
	jump strf1;
strf0:  ax0=513;			{ 512 data, + check sum }
strf1:  dm(tx_buf_len+1)=ax0;
	ax0=dm(cmd_buf+2);
	dm(power_exp)=ax0;		{ 2nd byte shift R power, >0 }
	mr0=dm(cmd_buf+3);		{ 3rd & 4th bytes are log_offset }
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(log_offset)=mr0;
	jump cmd_end;

{ 'E' Sets the delay of the right channel for time diversity binaural.
  Byte 1 & 2 are delay (1,2048) where the units are 1/48000 sec = 20.833 usec.
  Set delay=1 for no delay.  }
set_delay_right:
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(delay_right)=mr0;
	ax0=^delay;
	dm(del_i0_sav)=ax0;		{ Reset pointer to keep in proper area}
	jump cmd_end;

{ 'F' Sets the various parameters of the FFT.
  Byte 1 & 2, bit definitions:
	0	1.2 kHz input, 2.4 kHz sample rate
	1	2.4 kHz input, 4.8 kHz sample rate
	2	4.8 kHz input, 9.6 kHz sample rate
	3	48 kHz  sample rate (future)
	4	256 point (future)
	5	512 point (future)
	6	1024 point
	7	0=2.8 kHz I-Q, 1=4 kHz; I-F filters
	8	1=Stop current data collection, and clear pwr integrator.
		  Use with bit 10 software trigger to stop current collection.
	9	Triggered 0=Free run, 1=trig per bits 10 & 11
	10	Software trigger, start on 1 (reset by  collection fcn )
	11	Trig source 0=bit 10, 1=hardware PF0
	12	0=3 data sets per 2 sec, 1=1 data set per 2 sec
	13      0=2.8 kHz Interp 1=4 kHz
	14	0=normal, 1=no FFT, used for commands such as '4' and '5'
	15	nu
  Byte 3:
        0, 1    0  ( filled later in byte 14 of status for sequence, (0,2)
        2-7     Time tag, seconds since top of minute
   }
set_fft_fcn:
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);		{ mr0 is fft_fcn }
	dm(fft_fcn)=mr0;
	ar=dm(cmd_buf+3);
	dm(time_tag)=ar;
	jump cmd_end;

{ LMS_set - The beta 'adaptation gain' and decay for the denoise
  and auto notch.
  The MSB of the 2nd and 4th bytes are the delay line length.
}
LMS_set:
	mx0=dm(cmd_buf+1);		{ sqrt(LMS denoiser beta) }
	my0=mx0;
	mr=mx0*my0 (UU);		{ decompress data }
	sr=ashift mr0 by -1 (hi);	{ Divide by 2 }
	dm(beta)=sr1;

	mr0=dm(cmd_buf+2);
	ay0=0X00FE;			{ decay is in bits 1 to 7 }
	ar=mr0 and ay0;
	ay0=ar;
	ar=ar+ay0;			{ x2  range now (0,512) steps of 4 }
	ay0=32735;
	ar=ay0-ar;			{ range now (32735, 32238) (.999,.984)}
	dm(decay)=ar;

	ay0=0X0001;
	ar=mr0 and ay0;
	ay0=31;				{ Possible values: 31, 61 }
	if eq jump lmss1;		{ 0 -> delay=31 }
	ay0=61;				{ 1 -> delay=61 }
lmss1:	dm(denoise_length)=ay0;		{ Save for real time routine }
	ar=ay0-1;
	dm(denoise_lengthm1)=ar;	{ This saves interrupt calculations }

	mr0=dm(cmd_buf+3);		{ LMS auto-notch beta }
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(beta_aut)=mr0;
	jump cmd_end;

set_agc:				{ 'V' Make 3&4 0 to turn AGC off }
	{ First two bytes not used }
	mr0=dm(cmd_buf+3);		{ AGC Decay constant }
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(agc_decay)=mr0;
	jump cmd_end;

window_cmd:				{ 'W' 0=No window 1=Hamming 2=B-h
					      3=Tukey-0.25
					      Byte 2 is 0 for 2.8 kHz I-Q LPF
					      and 1 for 4 kHz  }
	ax0=dm(cmd_buf+1);
	dm(do_window)=ax0;
	ax0=dm(cmd_buf+2);
	dm(iq_lpf)=ax0;
	jump cmd_end;

set_mode:
	ax0=dm(cmd_buf+1);		{ 0=sideband, 1=cw, 2=fm, ... }
	dm(mode_select)=ax0;
	{ Tucked away in byte 2 is the shift number for mike gain (ver 2.5) }
	ax0=dm(cmd_buf+2);		{ 0=lsb, 1=ssb, 2=i-q binaural; 2 bits}
	ay0=0x0003;
	ar=ax0 and ay0;
	dm(ssb_select)=ar;
	{ Bits 6 and 7 are the shift amount for mike gain after shift right 6 }
	ay0=0x00d0;
	ar=ax0 and ay0;
	sr=lshift ar by -6 (hi);
	dm(mgain_shift)=sr1;
	mr0=dm(cmd_buf+3);		{ 3rd & 4th bytes are mike_gain }
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(mike_gain)=mr0;
	{ Following is only needed for FM to prevent noise burst at start
	  but harmless in other modes }
	ax0=32767;
	dm(squelch_noise)=ax0;		{ Decay back from here }
	jump cmd_end;

	{ set_noise - 'N' For use as a test signal generator there is a
	  random noise generator that can be added to the CW signal }
set_noise:
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(random_lev)=mr0;
	jump cmd_end;

	{ do_reset  -  'R' Reset functions }
	{ The variable need_reset is the first byte of the cmd and
	  is bit by bit read as follows:
	    Bit 0:  call local_init
	    Bit 1:  call init_util
	    Bit 2:  call LMS_init
	    Bit 3:  call prog19mhz
	    Bit 4:  jump fresh_start
	    Bit 5:  immediate fresh_start, the previous 5 were from background
	}
do_reset:
	ax0=dm(cmd_buf+1);
	af=tstbit 5 of ax0;
	if ne jump fresh_start;
	dm(need_reset)=ax0;
	jump cmd_end;

set_squelch:			{ 'S' 16 bit Squelch setting }
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(squelch)=mr0;
	{ Sqrt(dev) sent in buf+3. 114^2=12996 -> 5kHz dev. (181 max) }
	mx0=dm(cmd_buf+3);
	my0=mx0;
	mr=mx0*my0 (UU);	{ FM Deviation, Rev 2.5 }
	dm(fm_dev)=mr0;
	jump cmd_end;

set_t_r:
	ax0 = dm(cmd_buf+1);		{ Bit 0:  0=R, 1=T }
					{ Bit 1:  0=No Xmit status, 1=Send it }
	dm(t_r_control) = ax0;		{ Bit 4:  0=CW off, 1=CW on }
	mr0=dm(cmd_buf+3);		{ 3rd & 4th bytes are xmit_power }
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(xmit_power)=mr0;
	ax0=dm(t_r_control);
	af=tstbit 0 of ax0;
	if eq jump cmd_end;		{ Leave if receive }
	{Following is to initiate transmit:   }
	ax0=0;
	dm(get_data_block)=ax0;		{ Force an exit from get_data() }
	ax0=Nov2;
	dm(fft_buff_count)=ax0;		{ Get ready for next time }
	jump cmd_end;

set_af_gain:				{ 10 command,  16 bit AF gain }
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(af_gain)=mr0;
	mx0=dm(cmd_buf+3);		{ 8 bit shift to go to SE }
	dm(af_shift)=mx0;
	jump cmd_end;

set_r_if_gain:				{ 'I' 16 bit i-f gain for receive }
	mr0=dm(cmd_buf+1);		{ r_if_gain }
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(r_if_gain)=mr0;
	jump cmd_end;

set_sidetone:				{ 'H' 16 diff phase and amplitude }
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	dm(sdtone_dph)=mr0;

	mr0=dm(cmd_buf+3);
	mx0=dm(cmd_buf+4);
	mr=mr+mx0*my0 (UU);		{ my0 still 128 }
	dm(sdtone_ampl)=mr0;
	jump cmd_end;


	{ The second byte of this cmd is to steer the outputs so that
	  the receive audio goes to Left DAC and the transmit i-f
	  goes to the Right DAC.  For generating audio output signals
	  by using cw and lowering the frequency into the audio range.

	  Byte 3 is for "System" commands. Bit 0 is implemented!
	  If Bit 0==1 Monitor commands will be intercepted by the DSP.
	  Rev2.9 Removed this "feature." Not used and trouble.

	  Historically, there was a difference for inputs for audio
	  processor, that is no more. Thus the name.
	}
set_in_sel:
	ax0=dm(cmd_buf+2);
	dm(out_steer)=ax0;		{ 0==>L&R audio, 1==> L=Rcvr, R=T }
 {	ax0=dm(cmd_buf+3);
	dm(sys_ctrl)=ax0;  OUT }
	jump cmd_end;

 { 'K'  Parameters for voltage-to-frequency converter, monitors levels by ear.
   The byte in cmd_buf+1, dm(do_v2f) controls the load according to:
     dm(do_v2f):Bit 0=1 Use v2f & block reg audio
		Bit 1=0 Noise signal source;
		     =1 FFT Bin signal source
		Bit 2=0 Command word for do_v2f using bits 1, 2 and 7
		     =1 Load 16-bit data according to value of bits 3, 4, 5, 6:
		    76543210
		    x0000xxx  Load omt_v2f_h
		    x0001xxx  Load omt_v2f_l
		    x0010xxx  Load t_v2f_h
		    x0011xxx  Load t_v2f_l
		    x0100xxx  Load offset_v2f 
		    x0101xxx  Load k_v2f
		    x0110xxx  Load basef_v2f;
		    x0111xxx  Load lower bin for FFT v2f_bin_low
		    x1000xxx  Load upper bin for FFT v2f_bin_high
		    x1001xxx  Load v2f_af_level, the af level of the tone
		Bit 7=0   FFT Bin average (Bit 1 must be 1 )
		     =1   FFT Bin peak
  }
set_p2t:
	ax0=dm(cmd_buf+1);		{ The control byte }
	ar=tstbit 2 of ax0;		{ Load or control? }
	if gt jump sp2t_b2;		{ It is a data load, not process ctrl}
	dm(do_v2f)=ax0;			{ Bits 0, 1 and 7 are meaningful }
	jump cmd_end;			{ Control is now set }

sp2t_b2: mr0=dm(cmd_buf+3);		{ For the 3/4 pair, LS byte is 3 }
	mx0=dm(cmd_buf+4);		{ 4 is the MS byte }
	my0=128;			{ (UU) includes a x2, so this really}
	mr=mr+mx0*my0 (UU);		{   is a multiply by 256.    }
	{ Leave the 16 bits of data in mr0 and find where to put it }
	ay0=0X0078;			{ Mask off bits 3, 4, 5, 6 }
	ar=ax0 and ay0;
	if eq jump sp2t_0;		{ omt_v2f_h load (half of 32 bits) }
	ay0=0X0008;			{ Decrement value }
	ar=ar-ay0;
	if eq jump sp2t_1;		{ omt_v2f_l load }
	ar=ar-ay0;
	if eq jump sp2t_2;		{ t_v2f_h load (half of 32 bits) }
	ar=ar-ay0;
	if eq jump sp2t_3;		{ t_v2f_l load }
	ar=ar-ay0;
	if eq jump sp2t_4;		{ offset_v2f load }
	ar=ar-ay0;
	if eq jump sp2t_5;		{ k_v2f load }
	ar=ar-ay0;
	if eq jump sp2t_6;		{ basef_v2f load }
	ar=ar-ay0;
	if eq jump sp2t_7;		{ v2f_bin_low load }
	ar=ar-ay0;
	if eq jump sp2t_8;		{ v2f_bin_high load }
	ar=ar-ay0;
	if eq jump sp2t_9;		{ v2f_af_level }
	jump cmd_end;			{ Mistake protection }

sp2t_0:	dm(omt_v2f_h)=mr0;		{ Unexplained: word order reversed }
	jump cmd_end;			{  from byte order. (programmer old) }
sp2t_1:	dm(omt_v2f_l)=mr0;
	jump cmd_end;
sp2t_2:	dm(t_v2f_h)=mr0;
	jump cmd_end;
sp2t_3:	dm(t_v2f_l)=mr0;
	jump cmd_end;
sp2t_4:	dm(offset_v2f)=mr0;
	jump cmd_end;
sp2t_5:	dm(k_v2f)=mr0;
	jump cmd_end;
sp2t_6:	dm(basef_v2f)=mr0;
	jump cmd_end;
sp2t_7:	dm(v2f_bin_low)=mr0;
	jump cmd_end;
sp2t_8:	dm(v2f_bin_high)=mr0;
	jump cmd_end;
sp2t_9:	dm(v2f_af_level)=mr0;
	jump cmd_end;

 { 'X'  Parameters for FM tone generation.
   The byte in cmd_buf+1, dm(ttn_on) controls the function plus the data load:
                Bit 0=Use tone 1 in FM transmit
		Bit 1=Add in tone 2
		Bit 2=Add in microphone (must be 1 if bits 0 and 1 are both 0)
		Bit 6 NU
		Bit 7=0 Command word for ttn_on using bits 0, 1, 2
		     =1 Load 16-bit data according to value of bits 3, 4, 5:
		    76543210
		    1x000xxx  Load ttn_freq1
		    1x001xxx  Load ttn_freq2
		    1x010xxx  Load ttn_level1
		    1x011xxx  Load ttn_level2
 }
ttone_cmd:				{ 'X' transmit audio tone }
	ax0=dm(cmd_buf+1);		{ The control byte }
	ar=tstbit 7 of ax0;		{ Load or control? }
	if gt jump ttn_b2;		{ It is a data load, not process ctrl}
	dm(ttn_on)=ax0;			{ Bits 0, 1, 2 and 7 are meaningful }
	jump cmd_end;			{ Control is now set }

ttn_b2: mr0=dm(cmd_buf+3);		{ For the 3/4 pair, LS byte is 3 }
	mx0=dm(cmd_buf+4);		{ 4 is the MS byte }
	my0=128;			{ (UU) includes a x2, so this really}
	mr=mr+mx0*my0 (UU);		{   is a multiply by 256.    }
	{ Leave the 16 bits of data in mr0 and find where to put it }
	ay0=0X0038;			{ Mask off bits 3, 4, 5 }
	ar=ax0 and ay0;
	if eq jump ttn_0;		{ ttn_freq1 load }
	ay0=0X0008;			{ Decrement value }
	ar=ar-ay0;
	if eq jump ttn_1;		{ ttn_freq2 load }
	ar=ar-ay0;
	if eq jump ttn_2;		{ ttn_level1 load }
	ar=ar-ay0;
	if eq jump ttn_3;		{ ttn_level2 load }
	{ Room left here for 4 more growth }
	jump cmd_end;			{ Mistake protection }

ttn_0:	dm(ttn_freq1)=mr0;		{ Unexplained: word order reversed }
	jump cmd_end;			{  from byte order. (programmer old) }
ttn_1:	dm(ttn_freq2)=mr0;
	jump cmd_end;
ttn_2:	dm(ttn_level1)=mr0;
	jump cmd_end;
ttn_3:	dm(ttn_level2)=mr0;
	jump cmd_end;

get_cmd_data:
	mr0=dm(cmd_buf+1);
	mx0=dm(cmd_buf+2);
	my0=128;
	mr=mr+mx0*my0 (UU);
	i4=mr0;
	mr0=dm(cmd_buf+3);
	mx0=dm(cmd_buf+4);
	my0=128;
	mr=mr+mx0*my0 (UU);
	rts;

{*************************  RESET_HERE  ******************************
 * If the pc requests any form of reset, the variable need_reset is made
 * non-zero. The bits determine what reset functions will be done.
 * This is called in the background to do the reset.
 **********************************************************************}
reset_here:
	dis ints;
	ax0=dm(need_reset);
	ar=tstbit 0 of ax0;
	if ne call local_init;

	ax0=dm(need_reset);
	ar=tstbit 1 of ax0;
	if ne call init_util;

	ax0=dm(need_reset);
	ar=tstbit 2 of ax0;
	if ne call LMS_init;

	ax0=dm(need_reset);
	ar=tstbit 3 of ax0;
	if ne call prog19mhz;

	ax0=dm(need_reset);
	ar=tstbit 4 of ax0;
	if ne jump fresh_start;		{ Let them enable ints }

	ax0=0;
	dm(need_reset)=ax0;		{ Don't do it again }
	ena ints;
	rts;

	{ fresh_start() can be jumped to from anywhere.
	  It does a full restart of the DSP-10 process. }
fresh_start:
	imask=b#0000000000;
        dis sec_reg;
	pop sts; pop sts; pop sts; pop sts;
	pop cntr; pop cntr; pop cntr; pop cntr;
	pop pc; pop pc; pop pc; pop pc;
	pop loop; pop loop; pop loop; pop loop;
	jump start;

{ Routine to program Motorola 145170A synthesizer for 19.68 MHz LO.
  cntr determines number of bits being programmed which in turn in the 145170
  determines which register is to be programmed.  sr0 is the data to be
  programmed, MSB first. Called by prog19mhz.        RSL 5 May 98
  For EZ-Kit, added PF1 in parallel with PF6 in case that line was blown by
  reversing P101 on the DSP-10 board.          RSL 24 Sept 05
}
prog19: 
	ar=dm(PFDATA);			{ Get ready to work on it }
	ar=clrbit 6 of ar;		{ Clear bit 6 enables serial 145170 }
#if EZKIT
	ar=clrbit 1 of ar;
#endif
	dm(PFDATA)=ar;
	do pr19loop until ce;
	    af=tstbit 15 of sr0;	{ Get data bit to status }
	    sr=lshift sr0 by 1 (lo);	{ Move bits over, status uneffected }
	    if lt jump pr19a;		{ Data line to be a 1 }
	    reset FL0;			{ Here for data line=0 }
	    jump pr19b;			{ Go clock it out }
pr19a:	    set FL0;			{ Data line=1 }
pr19b:	    call delay1;		{ Setup data with rfi filters }
	    set fl2;			{ Clock in data }
	    call delay1;
	    reset fl2;			{ End clock pulse }
	    call delay1;
pr19loop:   nop;
	ar=dm(PFDATA);			{ Get ready to work on it }
	ar=setbit 6 of ar;		{ Set bit 6 un-enables serial 145170 }
#if EZKIT
	ar=setbit 1 of ar;
#endif
	dm(PFDATA)=ar;
	rts;

delay1:					{ Delay 10 microsecond }
	cntr=330;
	do dylp until ce;
dylp:	   nop;
	rts;

local_init:
	ax0=0;
	dm(cur_state) = ax0;
	dm(q_correction) = ax0;
	dm(audio_LMS_on) = ax0;		{ LMS is off }
	dm(out_steer)=ax0;		{ T&R to L&R }
	dm(data_format)=ax0;		{ 8 bit to host }
	dm(log_offset)=ax0;
	dm(squelch)=ax0;		{ FM squelch closed }
	dm(af_shift)=ax0;
	dm(audio_env)=ax0;
	dm(power_exp)=ax0;		{ Full power gain }
	dm(serial_error)=ax0;		{ No error }
	dm(random_lev)=ax0;		{ No noise on transmit }
	dm(need_gomon)=ax0;
	dm(af_gain)=ax0;
	dm(do_v2f)=ax0;			{ No v2f tone }
	dm(getn_data)=ax0;		{ Not getting data }
	ax0=12996;
	dm(fm_dev)=ax0;
	ax0=-1;
	dm(serial_timeout)=ax0;		{ No timeout, or bytes }
	ax0=1;
	dm(divide_count)=ax0;		{ For 2400, 4800 sample rate FFT }
	dm(data_collect)=ax0;		{ For non-triggered }
	ax0=0X0002;			{ Receive, send Xmit status, key up }
	dm(t_r_control) = ax0;		{ Receive mode }
	ax0=15;
	dm(agc_decay)=ax0;
	ax0=256;
	dm(agc_gain)=ax0;
	ax0=32767;			{ Full gain }
	dm(r_if_gain)=ax0;
	ax0=1000;
	dm(xmit_power)=ax0;
	ax0=0X5555;			{ dphase for 16 kHz }
	dm(if_dphase)=ax0;
	ax0=0X333;			{ dphase for 600 Hz }
	dm(sdtone_dph)=ax0;		{ CW sidetone }
	dm(ttn_freq1)=ax0;
	dm(ttn_freq2)=ax0;
	dm(ttn_level1)=ax0;
	dm(ttn_level1)=ax0;
	ax0=4;
	dm(ttn_on)=ax0;			{ Have voice on, no tones }
	ax0=0XCCD;			{ Ampl down 20 dB }
	dm(sdtone_ampl)=ax0;
	dm(v2f_af_level)=ax0;
	ax0=256;
	dm(delay_right)=ax0;
	ax0=^delay;
	dm(del_i0_sav)=ax0;
	ax0=0X0044;			{ 1024 pt, non-trig, 9.6 kHz, }
	dm(fft_fcn)=ax0;		{   FFT w/ 8 bit log data }

#if EZKIT 
	{ PF7, 6, 1 set out PF5 to 0=in }
	ax0=b#0111111111000010;
#else
	{ PF7, 6 set out PF5 to 0=in }
	ax0=b#0111111111000000;
#endif
	dm(PFTYPE)=ax0;			{ Program PF direction }

	ax0=3072;			{ 450 hz=3072*(4800/32768) }
	dm(basef_v2f)=ax0;
	rts;

#if EPROM
	{ GO_TO_MON 
	  If a command '$$$' or '$OK' is received, the DSP-10 routines stop,
	  and off we go to the monitor. The monitor init puts DSP-10
	  hardware into receive (if not already). AR has the monitor command.
	  all stacks, etc are restored in monitor init. dm(need_gomon) delays
	  the actual jump until out of the ISR.
	  This allows any EZ-Kit program to be run.  RSL v2.1 
	}
go_to_mon:
	dm(ar_value)=ar;
	ax0=1;
	dm(need_gomon)=ax0;
	jump bl5ac;

	{ ISR for IRQE. Executes when in background.
	  An alternate entry to the monitor. IRQE switch stops everything,
	  regardless of the state of sys_ctrl. RSL v2.1
	}
doirqe:
	ar=0;				{ Tell monitor we came from switch }
	dm(ar_value)=ar;
	ar=1;				{ Causes jump when in non-ISR }
	dm(need_gomon)=ar;
	rti;				{ Jump tomonitor after ISR }
#endif

prog19mhz:
	ar=dm(PFDATA);			{ Get ready to work on it }
	ar=setbit 6 of ar;		{ Set bit 6 un-enables serial 145170 }
#if EZKIT
	ar=setbit 1 of ar;
#endif
	dm(PFDATA)=ar;
	call delay1;
	{ Initialize the 145170 synthesizer: }
	reset FL0;			{ Data line to 0 }
	cntr=4;				{ 1st half reset 145170 }
	do init19loop until ce;
	    call delay1;		{ Setup data with rfi filters }
	    set FL2;			{ Clock in data }
	    call delay1;
	    reset FL2;			{ End clock pulse }
init19loop: nop;
	call delay1;
	sr0=0X1000; cntr=5;		{ 2nd half of reset 145170 }
	call prog19;
	call delay1;
	{ C Reg: C7=1       (Invert PDout)
	         C6=1       (Use PD A )
	         C5=1       (Use LE )
	         C4=C3=C2=0 (Disable reference out)
	         C1=0       (Disable fv)
	         C0=0       (Disable fr)
        }
	sr0=0XE000; cntr=8;		{ C register }
	call prog19;
	call delay1;
	{R register: 1MHz: 0X64,    4MHz: 0X190,    5MHz: 0X1F4,
	   6.4MHz: 0X280,    10MHz: 0X3E8 }
	sr0=0X03E8;			{ R register, 10.0 MHz Xtal }
	cntr=15;
	call prog19;
	call delay1;
	{ Alternate: R-sr0=0X3E2; (R=497)         N-sr0=0X3D2; (N=978)  }
	sr0=0X03D8; cntr=16; 		{ N register, N=984 }
	call prog19;
	rts;

{ Wideband routines - alternate wideband processing.
  The distinction is that no decimation is used here.  This is not suitable
  for narrow band filters, but is faster for processes for "tuning the band",
  such as binaural experiments.
  RSL 15 March 1999
}
wis0:
	my0=dm(rx_buf+1);		{ Get i-f sample from SPORT0 (codec) }
	ar = my0;
	ar = abs ar;			{ Look for biggest input }
	ay1 = dm(ad_big);
	none = ar - ay1;
	if lt jump wis1;		{ Not as big as previous biggest }
	dm(ad_big) = ar;		{ Save for next time }
wis1:

#if 0
	ax0 = dm(t_r_control);		{ Bit 0 = 1 is xmt }
	af=tstbit 0 of ax0;
	if gt jump xmt_isr;		{ Do xmit functions }
#endif
dm(temp1)=my0;
	ax0 = dm(if_dphase);		{ Phase increment for lo }
	ay0 = dm(if_phase);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	ax0 = ar;
	dm(if_phase) = ar;		{ For next time }
	call sin;			{ Phase in ax0, Sin returned in AR }
	mr=ar*my0(SS);			{ I mixer }
	ax0=dm(audio_fir_select);
	af=tstbit 0 of ax0;
	if gt jump wfia;
	call wfir1i;			{ LPF; mr1 in , mr1 out }
	jump wfib;
wfia:	call wfir2i;
wfib:	dm(isave) = mr1;

	ax0 = dm(if_phase);
	ay0 = 16384;  ar = ax0+ay0;	{ 90 degrees for quadrature lo }
	ax0 = ar;  call sin;		{ Cos lo signal, sin() preserves my0 }
	my0 = dm(temp1);
	mr=ar*my0(SS);			{ Sig still in my0, Q mixer }
	ax0=dm(audio_fir_select);
	af=tstbit 0 of ax0;
	if gt jump wfqa;
	call wfir1q;			{ LPF; mr1 in , mr1 out }
	jump wfqb;
wfqa:	call wfir2q;
wfqb:	dm(qsave) = mr1;

	ay1 = dm(isave);		{ Q signal for Hilbert }
	{ hilbert4 - delay: ay1 in, ax1 out;  90 deg: mr1 in, mr1 out }
	call do_hilbert4;
	ay1 = mr1;
	ar = ax1 - ay1;			{ - = lsb }
	dm(lsb_save) = ar;
	ar = ax1 + ay1;			{ + = usb }
	dm(usb_save) = ar;

{ Now figure what goes to L or R based on ssb_select as follows:
	Bit 0	0=LSB		1=USB		(narrow-band)
	Bit 1	0=monaural	1=binaural
	Bit 2	0=LSB/USB	1=I-Q source
	Bit 3
	Bit 4	0=Reverse L/R	1=Normal L/R	(not implemented yet )
	Bit 7   0=Narrow Band	1=Wideband processing
}
	ax0 = dm(ssb_select);
	af = tstbit 2 of ax0;		/* I-Q or SSB ?  */
	if gt jump wis2;		/* For I-Q */
	{ Here we set LSB/USB sources }
	mr1 = dm(lsb_save);
	ax1 = dm(usb_save);
	jump wis3;
wis2:	{ Here we set up I-Q sources }
	mr1 = dm(isave);
	ax1 = dm(qsave);
wis3:
	{Left channel audio in mr1.  Use of audsav is dual; for fft and
	 save for possible monaural use. }
	dm(audsav) = mr1;
	my0 = dm(af_gain);
	mr = mr1 * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);
	{ Note tx_buf+1 happens to be same for 1847 and AIC23 }
	dm(tx_buf+1) = sr1;		{ Left Audio output }

	ax0 = dm(ssb_select);
	af = tstbit 1 of ax0;		{ 0=monaural, 1=binaural }
	if gt jump wis4;
	ax1 = dm(audsav);		{ Put Left ch audio into Right ch }
wis4:
	{ Delay of right channel by amount delay_right.  If set to 1 there is
	  no delay. }
	{ Right channel audio in ax1 }
	i0=dm(del_i0_sav); l0=dm(delay_right);
	m0=0;
	ax0=dm(i0, m0);			{ Output of delay line, temp save }
	m0=1;
	dm(i0, m0)=ax1;			{ Input to delay line from ax1 }
	dm(del_i0_sav)=i0;
	mr1=ax0;			{ mr1 is now the delayed Right audio }

	my0 = dm(af_gain);
	mr = mr1 * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);

#if CODEC_1847
	dm(tx_buf+2) = sr1;		{ Audio output }
#endif
#if CODEC_AIC23
	dm(tx_buf+0) = sr1;
#endif

	ar = abs sr1;			{ Hunt for largest magnitude output }
	ay1 = dm(dac_big);		{  to be sent to pc, Q only }
	none = ar - ay1;
	if lt jump wis5;		{ Not as big as previous biggest }
	dm(dac_big) = ar;		{ Save for next time }

wis5:	call uart_rcvr;			{ See if character is coming in }

	ay0 = 1;
	ar = dm(block_count);		{ Ranges from 0 to 4 }
	ar = ar + ay0;			{ Update to next block }
	dm(block_count) = ar;		{ Save for next interrupt, now 1 to 5 }
	ar = ar - ay0;			{ Test for current block }
	if eq jump end_blocks;
	ar = ar - ay0;
	if eq jump end_blocks;
	ar = ar - ay0;
	if eq jump bin_bl3;
	ar = ar - ay0;
	if eq jump bin_bl4;
	jump bin_bl5;			{ This resets block_count }

.endmod;
